@@ -1,21 +1,11 @@
 //
 // Copyright 2011-2015 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#include <uhd/utils/thread_priority.hpp>
+#include <uhd/utils/thread.hpp>
 #include <uhd/convert.hpp>
 #include <uhd/utils/safe_main.hpp>
 #include <uhd/usrp/multi_usrp.hpp>
@@ -23,32 +13,46 @@
 #include <boost/format.hpp>
 #include <boost/thread/thread.hpp>
 #include <boost/algorithm/string.hpp>
-#include <boost/lexical_cast.hpp>
-//#include <boost/atomic.hpp>
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <iostream>
 #include <complex>
 #include <cstdlib>
+#include <atomic>
+#include <chrono>
+#include <thread>
 
 namespace po = boost::program_options;
 
-const double CLOCK_TIMEOUT = 1000;  // 1000mS timeout for external clock locking
-const double INIT_DELAY    = 0.05;  // 50mS initial delay before transmit
-//typedef boost::atomic<bool>   atomic_bool;
-// We'll fake atomic bools for now, for more backward compat.
-// This is just an example, after all.
-typedef bool atomic_bool;
+namespace {
+    constexpr int64_t CLOCK_TIMEOUT = 1000;  // 1000mS timeout for external clock locking
+    constexpr float   INIT_DELAY    = 0.05;  // 50mS initial delay before transmit
+}
 
 /***********************************************************************
  * Test result variables
  **********************************************************************/
-unsigned long long num_overflows = 0;
-unsigned long long num_underflows = 0;
+unsigned long long num_overruns = 0;
+unsigned long long num_underruns = 0;
 unsigned long long num_rx_samps = 0;
 unsigned long long num_tx_samps = 0;
 unsigned long long num_dropped_samps = 0;
 unsigned long long num_seq_errors = 0;
+unsigned long long num_seqrx_errors = 0; // "D"s
 unsigned long long num_late_commands = 0;
-unsigned long long num_timeouts = 0;
+unsigned long long num_timeouts_rx = 0;
+unsigned long long num_timeouts_tx = 0;
+
+inline boost::posix_time::time_duration time_delta(const boost::posix_time::ptime &ref_time)
+{
+    return boost::posix_time::microsec_clock::local_time() - ref_time;
+}
+
+inline std::string time_delta_str(const boost::posix_time::ptime &ref_time)
+{
+    return boost::posix_time::to_simple_string(time_delta(ref_time));
+}
+
+#define NOW() (time_delta_str(start_time))
 
 /***********************************************************************
  * Benchmark RX Rate
@@ -58,14 +62,15 @@ void benchmark_rx_rate(
         const std::string &rx_cpu,
         uhd::rx_streamer::sptr rx_stream,
         bool random_nsamps,
-        atomic_bool& burst_timer_elapsed
+        const boost::posix_time::ptime &start_time,
+        std::atomic<bool>& burst_timer_elapsed
 ) {
     uhd::set_thread_priority_safe();
 
     //print pre-test summary
     std::cout << boost::format(
-        "Testing receive rate %f Msps on %u channels"
-    ) % (usrp->get_rx_rate()/1e6) % rx_stream->get_num_channels() << std::endl;
+        "[%s] Testing receive rate %f Msps on %u channels"
+    ) % NOW() % (usrp->get_rx_rate()/1e6) % rx_stream->get_num_channels() << std::endl;
 
     //setup variables and allocate buffer
     uhd::rx_metadata_t md;
@@ -83,7 +88,8 @@ void benchmark_rx_rate(
     cmd.stream_now = (buffs.size() == 1);
     rx_stream->issue_stream_cmd(cmd);
 
-    const float burst_pkt_time = std::max(0.100, (2 * max_samps_per_packet/rate));
+    const float burst_pkt_time =
+        std::max<float>(0.100f, (2 * max_samps_per_packet/rate));
     float recv_timeout = burst_pkt_time + INIT_DELAY;
 
     bool stop_called = false;
@@ -102,7 +108,7 @@ void benchmark_rx_rate(
             recv_timeout = burst_pkt_time;
         }
         catch (uhd::io_error &e) {
-            std::cerr << "Caught an IO exception. " << std::endl;
+            std::cerr << "[" << NOW() << "] Caught an IO exception. " << std::endl;
             std::cerr << e.what() << std::endl;
             return;
         }
@@ -110,12 +116,20 @@ void benchmark_rx_rate(
         //handle the error codes
         switch(md.error_code){
         case uhd::rx_metadata_t::ERROR_CODE_NONE:
-            if (had_an_overflow){
+            if (had_an_overflow) {
                 had_an_overflow = false;
-                num_dropped_samps += (md.time_spec - last_time).to_ticks(rate);
+                const long dropped_samps =
+                    (md.time_spec - last_time).to_ticks(rate);
+                if (dropped_samps < 0) {
+                    std::cerr
+                        << "[" << NOW() << "] Timestamp after overrun recovery "
+                           "ahead of error timestamp! Unable to calculate "
+                           "number of dropped samples."
+                           "(Delta: " << dropped_samps << " ticks)\n";
+                }
+                num_dropped_samps += std::max<long>(1, dropped_samps);
             }
-            if ((burst_timer_elapsed or stop_called) and md.end_of_burst)
-            {
+            if ((burst_timer_elapsed or stop_called) and md.end_of_burst) {
                 return;
             }
             break;
@@ -125,12 +139,16 @@ void benchmark_rx_rate(
             last_time = md.time_spec;
             had_an_overflow = true;
             // check out_of_sequence flag to see if it was a sequence error or overflow
-            if (!md.out_of_sequence)
-                num_overflows++;
+            if (!md.out_of_sequence) {
+                num_overruns++;
+            } else {
+                num_seqrx_errors++;
+                std::cerr << "[" << NOW() << "] Detected Rx sequence error." << std::endl;
+            }
             break;
 
         case uhd::rx_metadata_t::ERROR_CODE_LATE_COMMAND:
-            std::cerr << "Receiver error: " << md.strerror() << ", restart streaming..."<< std::endl;
+            std::cerr << "[" << NOW() << "] Receiver error: " << md.strerror() << ", restart streaming..."<< std::endl;
             num_late_commands++;
             // Radio core will be in the idle state. Issue stream command to restart streaming.
             cmd.time_spec = usrp->get_time_now() + uhd::time_spec_t(0.05);
@@ -142,14 +160,14 @@ void benchmark_rx_rate(
             if (burst_timer_elapsed) {
                 return;
             }
-            std::cerr << "Receiver error: " << md.strerror() << ", continuing..." << std::endl;
-            num_timeouts++;
+            std::cerr << "[" << NOW() << "] Receiver error: " << md.strerror() << ", continuing..." << std::endl;
+            num_timeouts_rx++;
             break;
 
             // Otherwise, it's an error
         default:
-            std::cerr << "Receiver error: " << md.strerror() << std::endl;
-            std::cerr << "Unexpected error on recv, continuing..." << std::endl;
+            std::cerr << "[" << NOW() << "] Receiver error: " << md.strerror() << std::endl;
+            std::cerr << "[" << NOW() << "] Unexpected error on recv, continuing..." << std::endl;
             break;
         }
     }
@@ -162,15 +180,16 @@ void benchmark_tx_rate(
         uhd::usrp::multi_usrp::sptr usrp,
         const std::string &tx_cpu,
         uhd::tx_streamer::sptr tx_stream,
-        atomic_bool& burst_timer_elapsed,
+        std::atomic<bool>& burst_timer_elapsed,
+        const boost::posix_time::ptime &start_time,
         bool random_nsamps=false
 ) {
     uhd::set_thread_priority_safe();
 
     //print pre-test summary
     std::cout << boost::format(
-        "Testing transmit rate %f Msps on %u channels"
-    ) % (usrp->get_tx_rate()/1e6) % tx_stream->get_num_channels() << std::endl;
+        "[%s] Testing transmit rate %f Msps on %u channels"
+    ) % NOW() % (usrp->get_tx_rate()/1e6) % tx_stream->get_num_channels() << std::endl;
 
     //setup variables and allocate buffer
     uhd::tx_metadata_t md;
@@ -185,7 +204,6 @@ void benchmark_tx_rate(
 
     if (random_nsamps) {
         std::srand((unsigned int)time(NULL));
-        //while (not burst_timer_elapsed.load(boost::memory_order_relaxed)) {
         while (not burst_timer_elapsed) {
             size_t total_num_samps = rand() % max_samps_per_packet;
             size_t num_acc_samps = 0;
@@ -199,9 +217,15 @@ void benchmark_tx_rate(
             }
         }
     } else {
-        //while (not burst_timer_elapsed.load(boost::memory_order_relaxed)) {
         while (not burst_timer_elapsed) {
-            num_tx_samps += tx_stream->send(buffs, max_samps_per_packet, md)*tx_stream->get_num_channels();
+            const size_t num_tx_samps_sent_now = tx_stream->send(buffs, max_samps_per_packet, md)*tx_stream->get_num_channels();
+            num_tx_samps += num_tx_samps_sent_now;
+            if (num_tx_samps_sent_now == 0) {
+                num_timeouts_tx++;
+                if ((num_timeouts_tx % 10000) == 1) {
+                    std::cerr << "[" << NOW() << "] Tx timeouts: " << num_timeouts_tx << std::endl;
+                }
+            }
             md.has_time_spec = false;
         }
     }
@@ -213,14 +237,14 @@ void benchmark_tx_rate(
 
 void benchmark_tx_rate_async_helper(
         uhd::tx_streamer::sptr tx_stream,
-        atomic_bool& burst_timer_elapsed
+        const boost::posix_time::ptime &start_time,
+        std::atomic<bool>& burst_timer_elapsed
 ) {
     //setup variables and allocate buffer
     uhd::async_metadata_t async_md;
     bool exit_flag = false;
 
     while (true) {
-        //if (burst_timer_elapsed.load(boost::memory_order_relaxed)) {
         if (burst_timer_elapsed) {
             exit_flag = true;
         }
@@ -238,7 +262,7 @@ void benchmark_tx_rate_async_helper(
 
         case uhd::async_metadata_t::EVENT_CODE_UNDERFLOW:
         case uhd::async_metadata_t::EVENT_CODE_UNDERFLOW_IN_PACKET:
-            num_underflows++;
+            num_underruns++;
             break;
 
         case uhd::async_metadata_t::EVENT_CODE_SEQ_ERROR:
@@ -247,7 +271,7 @@ void benchmark_tx_rate_async_helper(
             break;
 
         default:
-            std::cerr << "Event code: " << async_md.event_code << std::endl;
+            std::cerr << "[" << NOW() << "] Event code: " << async_md.event_code << std::endl;
             std::cerr << "Unexpected event on async recv, continuing..." << std::endl;
             break;
         }
@@ -270,7 +294,8 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
     std::string mode, ref, pps;
     std::string channel_list, rx_channel_list, tx_channel_list;
     bool random_nsamps = false;
-    atomic_bool burst_timer_elapsed(false);
+    std::atomic<bool> burst_timer_elapsed(false);
+    size_t overrun_threshold, underrun_threshold, drop_threshold, seq_threshold;
 
     //setup the program options
     po::options_description desc("Allowed options");
@@ -293,6 +318,14 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         ("channels", po::value<std::string>(&channel_list)->default_value("0"), "which channel(s) to use (specify \"0\", \"1\", \"0,1\", etc)")
         ("rx_channels", po::value<std::string>(&rx_channel_list), "which RX channel(s) to use (specify \"0\", \"1\", \"0,1\", etc)")
         ("tx_channels", po::value<std::string>(&tx_channel_list), "which TX channel(s) to use (specify \"0\", \"1\", \"0,1\", etc)")
+        ("overrun-threshold", po::value<size_t>(&overrun_threshold),
+         "Number of overruns (O) which will declare the benchmark a failure.")
+        ("underrun-threshold", po::value<size_t>(&underrun_threshold),
+         "Number of underruns (U) which will declare the benchmark a failure.")
+        ("drop-threshold", po::value<size_t>(&drop_threshold),
+         "Number of dropped packets (D) which will declare the benchmark a failure.")
+        ("seq-threshold", po::value<size_t>(&seq_threshold),
+         "Number of dropped packets (D) which will declare the benchmark a failure.")
     ;
     po::variables_map vm;
     po::store(po::parse_command_line(argc, argv, desc), vm);
@@ -332,7 +365,8 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         std::cerr << "*** Warning! ***" << std::endl;
         std::cerr << "Benchmark results will be inaccurate on USRP1 due to insufficient features.\n" << std::endl;
     }
-    std::cout << boost::format("Creating the usrp device with: %s...") % args << std::endl;
+    boost::posix_time::ptime start_time(boost::posix_time::microsec_clock::local_time());
+    std::cout << boost::format("[%s] Creating the usrp device with: %s...") % NOW() % args << std::endl;
     uhd::usrp::multi_usrp::sptr usrp = uhd::usrp::multi_usrp::make(args);
 
     //always select the subdevice first, the channel mapping affects the other settings
@@ -364,13 +398,15 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         if(ref != "internal") {
             std::cout << "Now confirming lock on clock signals..." << std::endl;
             bool is_locked = false;
-            boost::system_time end_time = boost::get_system_time() + boost::posix_time::milliseconds(CLOCK_TIMEOUT);
+            auto end_time =
+                boost::get_system_time() +
+                boost::posix_time::milliseconds(CLOCK_TIMEOUT);
             for (int i = 0; i < num_mboards; i++) {
                 if (ref == "mimo" and i == 0) continue;
                 while((is_locked = usrp->get_mboard_sensor("ref_locked",i).to_bool()) == false and
                             boost::get_system_time() < end_time )
                 {
-                    boost::this_thread::sleep(boost::posix_time::milliseconds(1));
+                    std::this_thread::sleep_for(std::chrono::milliseconds(1));
                 }
                 if (is_locked == false) {
                     std::cerr << "ERROR: Unable to confirm clock signal locked on board:" << i <<  std::endl;
@@ -406,11 +442,11 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
 
         boost::split(channel_strings, rx_channel_list, boost::is_any_of("\"',"));
         for (size_t ch = 0; ch < channel_strings.size(); ch++) {
-            size_t chan = boost::lexical_cast<int>(channel_strings[ch]);
+            size_t chan = std::stoul(channel_strings[ch]);
             if (chan >= usrp->get_rx_num_channels()) {
                 throw std::runtime_error("Invalid channel(s) specified.");
             } else {
-                rx_channel_nums.push_back(boost::lexical_cast<int>(channel_strings[ch]));
+                rx_channel_nums.push_back(std::stoul(channel_strings[ch]));
             }
         }
     }
@@ -423,26 +459,27 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
 
         boost::split(channel_strings, tx_channel_list, boost::is_any_of("\"',"));
         for (size_t ch = 0; ch < channel_strings.size(); ch++) {
-            size_t chan = boost::lexical_cast<int>(channel_strings[ch]);
+            size_t chan = std::stoul(channel_strings[ch]);
             if (chan >= usrp->get_tx_num_channels()) {
                 throw std::runtime_error("Invalid channel(s) specified.");
             } else {
-                tx_channel_nums.push_back(boost::lexical_cast<int>(channel_strings[ch]));
+                tx_channel_nums.push_back(std::stoul(channel_strings[ch]));
             }
         }
     }
 
-    std::cout << boost::format("Setting device timestamp to 0...") << std::endl;
-    const bool sync_channels =
-            pps == "mimo" or
-            ref == "mimo" or
-            rx_channel_nums.size() > 1 or
-            tx_channel_nums.size() > 1
-    ;
-    if (!sync_channels) {
-       usrp->set_time_now(0.0);
+    std::cout << boost::format("[%s] Setting device timestamp to 0...") % NOW() << std::endl;
+    if (pps == "mimo" or ref == "mimo") {
+        // only set the master's time, the slave's is automatically sync'd
+        usrp->set_time_now(uhd::time_spec_t(0.0), 0);
+        // ensure that the setter has completed
+        usrp->get_time_now();
+        // wait for the time to sync
+        std::this_thread::sleep_for(std::chrono::milliseconds(1));
+    } else if (rx_channel_nums.size() > 1 or tx_channel_nums.size() > 1) {
+        usrp->set_time_unknown_pps(uhd::time_spec_t(0.0));
     } else {
-       usrp->set_time_unknown_pps(uhd::time_spec_t(0.0));
+        usrp->set_time_now(0.0);
     }
 
     //spawn the receive test thread
@@ -452,7 +489,17 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         uhd::stream_args_t stream_args(rx_cpu, rx_otw);
         stream_args.channels = rx_channel_nums;
         uhd::rx_streamer::sptr rx_stream = usrp->get_rx_stream(stream_args);
-        thread_group.create_thread(boost::bind(&benchmark_rx_rate, usrp, rx_cpu, rx_stream, random_nsamps, boost::ref(burst_timer_elapsed)));
+        auto rx_thread = thread_group.create_thread([=, &burst_timer_elapsed](){
+            benchmark_rx_rate(
+                usrp,
+                rx_cpu,
+                rx_stream,
+                random_nsamps,
+                start_time,
+                burst_timer_elapsed
+            );
+        });
+        uhd::set_thread_name(rx_thread, "bmark_rx_stream");
     }
 
     //spawn the transmit test thread
@@ -462,41 +509,119 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         uhd::stream_args_t stream_args(tx_cpu, tx_otw);
         stream_args.channels = tx_channel_nums;
         uhd::tx_streamer::sptr tx_stream = usrp->get_tx_stream(stream_args);
-        thread_group.create_thread(boost::bind(&benchmark_tx_rate, usrp, tx_cpu, tx_stream, boost::ref(burst_timer_elapsed), random_nsamps));
-        thread_group.create_thread(boost::bind(&benchmark_tx_rate_async_helper, tx_stream, boost::ref(burst_timer_elapsed)));
+        auto tx_thread = thread_group.create_thread([=, &burst_timer_elapsed](){
+            benchmark_tx_rate(
+                usrp,
+                tx_cpu,
+                tx_stream,
+                burst_timer_elapsed,
+                start_time,
+                random_nsamps
+            );
+        });
+        uhd::set_thread_name(tx_thread, "bmark_tx_stream");
+        auto tx_async_thread = thread_group.create_thread([=, &burst_timer_elapsed](){
+            benchmark_tx_rate_async_helper(
+                tx_stream,
+                start_time,
+                burst_timer_elapsed
+            );
+        });
+        uhd::set_thread_name(tx_async_thread, "bmark_tx_helper");
     }
 
     //sleep for the required duration
-    const long secs = long(duration);
-    const long usecs = long((duration - secs)*1e6);
-    boost::this_thread::sleep(boost::posix_time::seconds(secs)
-            + boost::posix_time::microseconds(usecs)
-            + boost::posix_time::milliseconds( (rx_channel_nums.size() <= 1 and tx_channel_nums.size() <= 1) ? 0 : (INIT_DELAY * 1000))
+    const bool wait_for_multichan =
+        (rx_channel_nums.size() <= 1 and tx_channel_nums.size() <= 1);
+    const int64_t secs =
+        int64_t(duration + (wait_for_multichan ? 0 : INIT_DELAY * 1000));
+    const int64_t usecs = int64_t((duration - secs)*1e6);
+    std::this_thread::sleep_for(
+        std::chrono::seconds(secs) +
+        std::chrono::microseconds(usecs)
     );
 
     //interrupt and join the threads
-    //burst_timer_elapsed.store(true, boost::memory_order_relaxed);
     burst_timer_elapsed = true;
     thread_group.join_all();
 
-    //print summary
-    std::cout << std::endl << boost::format(
-        "Benchmark rate summary:\n"
-        "  Num received samples:    %u\n"
-        "  Num dropped samples:     %u\n"
-        "  Num overflows detected:  %u\n"
-        "  Num transmitted samples: %u\n"
-        "  Num sequence errors:     %u\n"
-        "  Num underflows detected: %u\n"
-        "  Num late commands:       %u\n"
-        "  Num timeouts:            %u\n"
-    ) % num_rx_samps % num_dropped_samps
-      % num_overflows % num_tx_samps
-      % num_seq_errors % num_underflows
-      % num_late_commands % num_timeouts
-      << std::endl;
+    std::cout << "[" << NOW() << "] Benchmark complete." << std::endl << std::endl;
 
+    //print summary
+    const std::string threshold_err(" ERROR: Exceeds threshold!");
+    const bool overrun_threshold_err =
+        vm.count("overrun-threshold") and
+        num_overruns > overrun_threshold;
+    const bool underrun_threshold_err =
+        vm.count("underrun-threshold") and
+        num_underruns > underrun_threshold;
+    const bool drop_threshold_err =
+        vm.count("drop-threshold") and
+        num_seqrx_errors > drop_threshold;
+    const bool seq_threshold_err =
+        vm.count("seq-threshold") and
+        num_seq_errors > seq_threshold;
+    std::cout << std::endl
+        << boost::format(
+                "Benchmark rate summary:\n"
+                "  Num received samples:     %u\n"
+                "  Num dropped samples:      %u\n"
+                "  Num overruns detected:    %u\n"
+                "  Num transmitted samples:  %u\n"
+                "  Num sequence errors (Tx): %u\n"
+                "  Num sequence errors (Rx): %u\n"
+                "  Num underruns detected:   %u\n"
+                "  Num late commands:        %u\n"
+                "  Num timeouts (Tx):        %u\n"
+                "  Num timeouts (Rx):        %u\n"
+            ) % num_rx_samps
+              % num_dropped_samps
+              % num_overruns
+              % num_tx_samps
+              % num_seq_errors
+              % num_seqrx_errors
+              % num_underruns
+              % num_late_commands
+              % num_timeouts_tx
+              % num_timeouts_rx
+        << std::endl;
     //finished
     std::cout << std::endl << "Done!" << std::endl << std::endl;
+
+    if (overrun_threshold_err
+            || underrun_threshold_err
+            || drop_threshold_err
+            || seq_threshold_err) {
+        std::cout << "The following error thresholds were exceeded:\n";
+        if (overrun_threshold_err) {
+            std::cout
+                << boost::format("  * Overruns (%d/%d)")
+                   % num_overruns
+                   % overrun_threshold
+                << std::endl;
+        }
+        if (underrun_threshold_err) {
+            std::cout
+                << boost::format("  * Underruns (%d/%d)")
+                   % num_underruns
+                   % underrun_threshold
+                << std::endl;
+        }
+        if (drop_threshold_err) {
+            std::cout
+                << boost::format("  * Dropped packets (RX) (%d/%d)")
+                   % num_seqrx_errors
+                   % drop_threshold
+                << std::endl;
+        }
+        if (seq_threshold_err) {
+            std::cout
+                << boost::format("  * Dropped packets (TX) (%d/%d)")
+                   % num_seq_errors
+                   % seq_threshold
+                << std::endl;
+        }
+        return EXIT_FAILURE;
+    }
     return EXIT_SUCCESS;
 }
