@@ -1,31 +1,22 @@
 //
 // Copyright 2012,2014,20160 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#include <uhd/utils/thread_priority.hpp>
+#include <uhd/utils/thread.hpp>
 #include <uhd/utils/safe_main.hpp>
 #include <uhd/usrp/multi_usrp.hpp>
 #include <boost/program_options.hpp>
 #include <boost/format.hpp>
-#include <boost/thread/thread.hpp>
 #include <boost/math/special_functions/round.hpp>
 #include <iostream>
 #include <complex>
 #include <utility>
 #include <vector>
+#include <chrono>
+#include <thread>
 
 static const double SAMP_RATE = 1e6;
 
@@ -102,7 +93,7 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
 
     std::cout << boost::format("\nTesting %s coercion...") % type << std::endl;
 
-    BOOST_FOREACH(const uhd::range_t &range, freq_ranges){
+    for(const uhd::range_t &range:  freq_ranges){
         double freq_begin = range.start();
         double freq_end = range.stop();
 
@@ -152,13 +143,13 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
 
     bool has_sensor = (std::find(dboard_sensor_names.begin(), dboard_sensor_names.end(), "lo_locked")) != dboard_sensor_names.end();
 
-    BOOST_FOREACH(double freq, freqs){
+    for(double freq:  freqs){
 
         //Testing for successful frequency tune
         if(type == "TX") usrp->set_tx_freq(freq,chan);
         else usrp->set_rx_freq(freq,chan);
 
-        boost::this_thread::sleep(boost::posix_time::microseconds(long(1000)));
+        std::this_thread::sleep_for(std::chrono::microseconds(long(1000)));
         double actual_freq = (type == "TX") ? usrp->get_tx_freq(chan)
                                             : usrp->get_rx_freq(chan);
 
@@ -195,7 +186,7 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
                 if (is_locked) {
                     break;
                 }
-                boost::this_thread::sleep(boost::posix_time::microseconds(1000));
+                std::this_thread::sleep_for(std::chrono::microseconds(1000));
             }
             if(is_locked){
                 if(verbose) std::cout << boost::format("LO successfully locked at %s frequency %s.")
@@ -212,11 +203,11 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
 
             //Testing for successful gain tune
 
-            BOOST_FOREACH(double gain, gains){
+            for(double gain:  gains){
                 if(type == "TX") usrp->set_tx_gain(gain,chan);
                 else usrp->set_rx_gain(gain,chan);
 
-                boost::this_thread::sleep(boost::posix_time::microseconds(1000));
+                std::this_thread::sleep_for(std::chrono::microseconds(1000));
 
                 double actual_gain = (type == "TX") ? usrp->get_tx_gain(chan)
                                                     : usrp->get_rx_gain(chan);
@@ -266,7 +257,7 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
     }
     else{
         results += "USRP did not successfully tune to the following frequencies: ";
-        BOOST_FOREACH(double bad_freq, bad_tune_freqs){
+        for(double bad_freq:  bad_tune_freqs){
             if(bad_freq != *bad_tune_freqs.begin()) results += ", ";
             results += MHz_str(bad_freq);
         }
@@ -282,7 +273,7 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
         }
         else{
             results += "LO did not lock at the following frequencies: ";
-            BOOST_FOREACH(double bad_freq, no_lock_freqs){
+            for(double bad_freq:  no_lock_freqs){
                 if(bad_freq != *no_lock_freqs.begin()) results += ", ";
                 results += MHz_str(bad_freq);
             }
@@ -298,7 +289,7 @@ std::string coercion_test(uhd::usrp::multi_usrp::sptr usrp, std::string type, in
         }
         else{
             results += "USRP did not successfully set gain under the following circumstances:";
-            BOOST_FOREACH(double_pair bad_pair, bad_gain_vals){
+            for(double_pair bad_pair:  bad_gain_vals){
                 double bad_freq = bad_pair.first;
                 double bad_gain = bad_pair.second;
                 results += str(boost::format("\nFrequency: %s, Gain: %5.2f") % MHz_str(bad_freq) % bad_gain);
@@ -420,7 +411,7 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
 
     //Setting clock source
     usrp->set_clock_source(ref);
-    boost::this_thread::sleep(boost::posix_time::seconds(1));
+    std::this_thread::sleep_for(std::chrono::seconds(1));
 
     std::vector<std::string> sensor_names = usrp->get_mboard_sensor_names(0);
     if ((ref == "mimo") and (std::find(sensor_names.begin(), sensor_names.end(), "mimo_locked") != sensor_names.end())) {
