@@ -1,31 +1,22 @@
 //
 // Copyright 2011-2012,2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include <uhd/types/tune_request.hpp>
-#include <uhd/utils/thread_priority.hpp>
+#include <uhd/utils/thread.hpp>
 #include <uhd/utils/safe_main.hpp>
 #include <uhd/usrp/multi_usrp.hpp>
 #include <boost/program_options.hpp>
 #include <boost/format.hpp>
-#include <boost/thread.hpp>
 #include <iostream>
 #include <fstream>
 #include <complex>
 #include <csignal>
+#include <chrono>
+#include <thread>
 
 namespace po = boost::program_options;
 
@@ -33,17 +24,11 @@ static bool stop_signal_called = false;
 void sig_int_handler(int){stop_signal_called = true;}
 
 template<typename samp_type> void send_from_file(
-    uhd::usrp::multi_usrp::sptr usrp,
-    const std::string &cpu_format,
-    const std::string &wire_format,
+    uhd::tx_streamer::sptr tx_stream,
     const std::string &file,
     size_t samps_per_buff
 ){
 
-    //create a transmit streamer
-    uhd::stream_args_t stream_args(cpu_format, wire_format);
-    uhd::tx_streamer::sptr tx_stream = usrp->get_tx_stream(stream_args);
-
     uhd::tx_metadata_t md;
     md.start_of_burst = false;
     md.end_of_burst = false;
@@ -69,7 +54,7 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
     uhd::set_thread_priority_safe();
 
     //variables to be set by po
-    std::string args, file, type, ant, subdev, ref, wirefmt;
+    std::string args, file, type, ant, subdev, ref, wirefmt, channel;
     size_t spb;
     double rate, freq, gain, bw, delay, lo_off;
 
@@ -90,7 +75,8 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         ("bw", po::value<double>(&bw), "analog frontend filter bandwidth in Hz")
         ("ref", po::value<std::string>(&ref)->default_value("internal"), "reference source (internal, external, mimo)")
         ("wirefmt", po::value<std::string>(&wirefmt)->default_value("sc16"), "wire format (sc8 or sc16)")
-        ("delay", po::value<double>(&delay)->default_value(0.0), "specify a delay between repeated transmission of file")
+        ("delay", po::value<double>(&delay)->default_value(0.0), "specify a delay between repeated transmission of file (in seconds)")
+        ("channel", po::value<std::string>(&channel)->default_value("0"), "which channel to use")
         ("repeat", "repeatedly transmit file")
         ("int-n", "tune USRP with integer-n tuning")
     ;
@@ -150,15 +136,20 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
 
     //set the analog frontend filter bandwidth
     if (vm.count("bw")){
-        std::cout << boost::format("Setting TX Bandwidth: %f MHz...") % bw << std::endl;
+        std::cout << boost::format("Setting TX Bandwidth: %f MHz...")
+                     % (bw / 1e6)
+                  << std::endl;
         usrp->set_tx_bandwidth(bw);
-        std::cout << boost::format("Actual TX Bandwidth: %f MHz...") % usrp->get_tx_bandwidth() << std::endl << std::endl;
+        std::cout << boost::format("Actual TX Bandwidth: %f MHz...")
+                     % (usrp->get_tx_bandwidth() / 1e6)
+                  << std::endl << std::endl;
     }
 
     //set the antenna
     if (vm.count("ant")) usrp->set_tx_antenna(ant);
 
-    boost::this_thread::sleep(boost::posix_time::seconds(1)); //allow for some setup time
+    //allow for some setup time:
+    std::this_thread::sleep_for(std::chrono::seconds(1));
 
     //Check Ref and LO Lock detect
     std::vector<std::string> sensor_names;
@@ -186,14 +177,29 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         std::cout << "Press Ctrl + C to stop streaming..." << std::endl;
     }
 
+    //create a transmit streamer
+    std::string cpu_format;
+    std::vector<size_t> channel_nums;
+    if (type == "double") cpu_format = "fc64";
+    else if (type == "float") cpu_format = "fc32";
+    else if (type == "short") cpu_format = "sc16";
+    uhd::stream_args_t stream_args(cpu_format, wirefmt);
+    channel_nums.push_back(boost::lexical_cast<size_t>(channel));
+    stream_args.channels = channel_nums;
+    uhd::tx_streamer::sptr tx_stream = usrp->get_tx_stream(stream_args);
+
     //send from file
     do{
-        if (type == "double") send_from_file<std::complex<double> >(usrp, "fc64", wirefmt, file, spb);
-        else if (type == "float") send_from_file<std::complex<float> >(usrp, "fc32", wirefmt, file, spb);
-        else if (type == "short") send_from_file<std::complex<short> >(usrp, "sc16", wirefmt, file, spb);
+        if (type == "double") send_from_file<std::complex<double> >(tx_stream, file, spb);
+        else if (type == "float") send_from_file<std::complex<float> >(tx_stream, file, spb);
+        else if (type == "short") send_from_file<std::complex<short> >(tx_stream, file, spb);
         else throw std::runtime_error("Unknown type " + type);
 
-        if(repeat and delay != 0.0) boost::this_thread::sleep(boost::posix_time::milliseconds(delay));
+        if(repeat and delay > 0.0) {
+            std::this_thread::sleep_for(
+                std::chrono::milliseconds(int64_t(delay*1000))
+            );
+        }
     } while(repeat and not stop_signal_called);
 
     //finished
