@@ -1,23 +1,13 @@
 //
 // Copyright 2010-2012,2014-2015 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "wavetable.hpp"
 #include <uhd/types/tune_request.hpp>
-#include <uhd/utils/thread_priority.hpp>
+#include <uhd/utils/thread.hpp>
 #include <uhd/utils/safe_main.hpp>
 #include <uhd/utils/static.hpp>
 #include <uhd/usrp/multi_usrp.hpp>
@@ -26,7 +16,6 @@
 #include <boost/program_options.hpp>
 #include <boost/math/special_functions/round.hpp>
 #include <boost/format.hpp>
-#include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string.hpp>
 #include <boost/filesystem.hpp>
 #include <iostream>
@@ -107,7 +96,7 @@ template<typename samp_type> void recv_to_file(
     const std::string &file,
     size_t samps_per_buff,
     int num_requested_samples,
-    float settling_time,
+    double settling_time,
     std::vector<size_t> rx_channel_nums
 ){
     int num_total_samps = 0;
@@ -137,7 +126,7 @@ template<typename samp_type> void recv_to_file(
     UHD_ASSERT_THROW(outfiles.size() == buffs.size());
     UHD_ASSERT_THROW(buffs.size() == rx_channel_nums.size());
     bool overflow_message = true;
-    float timeout = settling_time + 0.1f; //expected settling time + padding for first recv
+    double timeout = settling_time + 0.1f; //expected settling time + padding for first recv
 
     //setup streaming
     uhd::stream_cmd_t stream_cmd((num_requested_samples == 0)?
@@ -209,7 +198,7 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
     std::string rx_args, file, type, rx_ant, rx_subdev, rx_channels;
     size_t total_num_samps, spb;
     double rx_rate, rx_freq, rx_gain, rx_bw;
-    float settling;
+    double settling;
 
     //setup the program options
     po::options_description desc("Allowed options");
@@ -220,7 +209,7 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
         ("file", po::value<std::string>(&file)->default_value("usrp_samples.dat"), "name of the file to write binary samples to")
         ("type", po::value<std::string>(&type)->default_value("short"), "sample type in file: double, float, or short")
         ("nsamps", po::value<size_t>(&total_num_samps)->default_value(0), "total number of samples to receive")
-        ("settling", po::value<float>(&settling)->default_value(float(0.2)), "settling time (seconds) before receiving")
+        ("settling", po::value<double>(&settling)->default_value(double(0.2)), "settling time (seconds) before receiving")
         ("spb", po::value<size_t>(&spb)->default_value(0), "samples per buffer, 0 for default")
         ("tx-rate", po::value<double>(&tx_rate), "rate of transmit outgoing samples")
         ("rx-rate", po::value<double>(&rx_rate), "rate of receive incoming samples")
@@ -262,36 +251,36 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
     std::cout << boost::format("Creating the receive usrp device with: %s...") % rx_args << std::endl;
     uhd::usrp::multi_usrp::sptr rx_usrp = uhd::usrp::multi_usrp::make(rx_args);
 
+    //always select the subdevice first, the channel mapping affects the other settings
+    if (vm.count("tx-subdev")) tx_usrp->set_tx_subdev_spec(tx_subdev);
+    if (vm.count("rx-subdev")) rx_usrp->set_rx_subdev_spec(rx_subdev);
+
     //detect which channels to use
     std::vector<std::string> tx_channel_strings;
     std::vector<size_t> tx_channel_nums;
     boost::split(tx_channel_strings, tx_channels, boost::is_any_of("\"',"));
     for(size_t ch = 0; ch < tx_channel_strings.size(); ch++){
-        size_t chan = boost::lexical_cast<int>(tx_channel_strings[ch]);
+        size_t chan = std::stoi(tx_channel_strings[ch]);
         if(chan >= tx_usrp->get_tx_num_channels()){
             throw std::runtime_error("Invalid TX channel(s) specified.");
         }
-        else tx_channel_nums.push_back(boost::lexical_cast<int>(tx_channel_strings[ch]));
+        else tx_channel_nums.push_back(std::stoi(tx_channel_strings[ch]));
     }
     std::vector<std::string> rx_channel_strings;
     std::vector<size_t> rx_channel_nums;
     boost::split(rx_channel_strings, rx_channels, boost::is_any_of("\"',"));
     for(size_t ch = 0; ch < rx_channel_strings.size(); ch++){
-        size_t chan = boost::lexical_cast<int>(rx_channel_strings[ch]);
+        size_t chan = std::stoi(rx_channel_strings[ch]);
         if(chan >= rx_usrp->get_rx_num_channels()){
             throw std::runtime_error("Invalid RX channel(s) specified.");
         }
-        else rx_channel_nums.push_back(boost::lexical_cast<int>(rx_channel_strings[ch]));
+        else rx_channel_nums.push_back(std::stoi(rx_channel_strings[ch]));
     }
 
     //Lock mboard clocks
     tx_usrp->set_clock_source(ref);
     rx_usrp->set_clock_source(ref);
 
-    //always select the subdevice first, the channel mapping affects the other settings
-    if (vm.count("tx-subdev")) tx_usrp->set_tx_subdev_spec(tx_subdev);
-    if (vm.count("rx-subdev")) rx_usrp->set_rx_subdev_spec(rx_subdev);
-
     std::cout << boost::format("Using TX Device: %s") % tx_usrp->get_pp_string() << std::endl;
     std::cout << boost::format("Using RX Device: %s") % rx_usrp->get_pp_string() << std::endl;
 
@@ -378,9 +367,10 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
             rx_usrp->set_rx_bandwidth(rx_bw, channel);
             std::cout << boost::format("Actual RX Bandwidth: %f MHz...") % (rx_usrp->get_rx_bandwidth(channel)/1e6) << std::endl << std::endl;
         }
+
+        // set the receive antenna
+        if (vm.count("rx-ant")) rx_usrp->set_rx_antenna(rx_ant, channel);
     }
-    //set the receive antenna
-    if (vm.count("ant")) rx_usrp->set_rx_antenna(rx_ant);
 
     //for the const wave, set the wave freq for small samples per period
     if (wave_freq == 0 and wave_type == "CONST"){
@@ -416,7 +406,7 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
     md.start_of_burst = true;
     md.end_of_burst   = false;
     md.has_time_spec  = true;
-    md.time_spec = uhd::time_spec_t(0.1); //give us 0.1 seconds to fill the tx buffers
+    md.time_spec = uhd::time_spec_t(0.5); //give us 0.5 seconds to fill the tx buffers
 
     //Check Ref and LO Lock detect
     std::vector<std::string> tx_sensor_names, rx_sensor_names;
