@@ -1,18 +1,8 @@
 //
 // Copyright 2013 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifndef INCLUDED_USRPRIO_RPC_CLIENT_HPP
@@ -55,8 +45,6 @@ public:
     nirio_status niusrprio_download_fpga_to_flash(
         NIUSRPRIO_DOWNLOAD_FPGA_TO_FLASH_ARGS);
 
-    static const int64_t DEFAULT_TIMEOUT_IN_MS = 5000;
-
 private:
     static nirio_status _boost_error_to_nirio_status(const boost::system::error_code& err);
 
