@@ -1,18 +1,8 @@
 //
 // Copyright 2010-2013,2015-2016 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifndef INCLUDED_UHD_USRP_DBOARD_IFACE_HPP
@@ -286,6 +276,10 @@ public:
         const uhd::usrp::fe_connection_t& fe_conn
     ) = 0;
 
+    /*! Returns the true if set_fe_connection() is implemented on this dboard_iface
+     */
+    virtual bool has_set_fe_connection(const unit_t) { return false; }
+
     /*!
      * Get the command time.
      * \return the command time
