@@ -40,7 +40,10 @@
 #include <string>
 #include <vector>
 
-namespace uhd{ namespace usrp{
+namespace uhd {
+    class device3;
+
+    namespace usrp{
 
 /*!
  * The Multi-USRP device class:
@@ -122,10 +125,24 @@ public:
      */
     virtual device::sptr get_device(void) = 0;
 
+    /*! Returns true if this is a generation-3 device.
+     */
+    virtual bool is_device3(void) = 0;
+
+    /*!
+     * Get the underlying device3 object. Only works for generation-3 (or later) devices.
+     *
+     * This is needed to get access to the streaming API and properties.
+     *
+     * \return The uhd::device3 object for this USRP.
+     * \throws uhd::type_error if this device is not actually a generation-3 device.
+     */
+    virtual boost::shared_ptr<uhd::device3> get_device3(void) = 0;
+
     //! Convenience method to get a RX streamer. See also uhd::device::get_rx_stream().
     virtual rx_streamer::sptr get_rx_stream(const stream_args_t &args) = 0;
 
-    //! Convenience method to get a TX streamer. See also uhd::device::get_rx_stream().
+    //! Convenience method to get a TX streamer. See also uhd::device::get_tx_stream().
     virtual tx_streamer::sptr get_tx_stream(const stream_args_t &args) = 0;
 
     /*!
@@ -1010,6 +1027,12 @@ public:
      */
     virtual void set_rx_dc_offset(const std::complex<double> &offset, size_t chan = ALL_CHANS) = 0;
 
+    /*!
+     * Get the valid range for RX DC offset values.
+     * \param chan the channel index 0 to N-1
+     */
+    virtual meta_range_t get_rx_dc_offset_range(size_t chan = ALL_CHANS) = 0;
+
     /*!
      * Enable/disable the automatic IQ imbalance correction.
      *
@@ -1293,6 +1316,12 @@ public:
      */
     virtual void set_tx_dc_offset(const std::complex<double> &offset, size_t chan = ALL_CHANS) = 0;
 
+    /*!
+     * Get the valid range for TX DC offset values.
+     * \param chan the channel index 0 to N-1
+     */
+    virtual meta_range_t get_tx_dc_offset_range(size_t chan = ALL_CHANS) = 0;
+
     /*!
      * Set the TX frontend IQ imbalance correction.
      * Use this to adjust the magnitude and phase of I and Q.
