@@ -1,18 +1,8 @@
 //
 // Copyright 2015-2016 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifndef INCLUDED_UHD_USRP_H
@@ -1138,23 +1128,6 @@ UHD_API uhd_error uhd_usrp_get_tx_sensor_names(
     uhd_string_vector_handle *sensor_names_out
 );
 
-//! Enable or disable TX DC offset correction for the given channel
-/*!
- * See uhd::usrp::multi_usrp::set_tx_dc_offset() for more details.
- */
-UHD_API uhd_error uhd_usrp_set_tx_dc_offset_enabled(
-    uhd_usrp_handle h,
-    bool enb,
-    size_t chan
-);
-
-//! Enable or disable TX IQ imbalance correction for the given channel
-UHD_API uhd_error uhd_usrp_set_tx_iq_balance_enabled(
-    uhd_usrp_handle h,
-    bool enb,
-    size_t chan
-);
-
 /****************************************************************************
  * GPIO methods
  ***************************************************************************/
