@@ -1,18 +1,8 @@
 //
 // Copyright 2011,2016 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include <uhd/convert.hpp>
@@ -22,7 +12,6 @@
 #include <uhd/exception.hpp>
 #include <stdint.h>
 #include <boost/format.hpp>
-#include <boost/foreach.hpp>
 #include <complex>
 
 using namespace uhd;
@@ -81,10 +70,7 @@ void uhd::convert::register_converter(
     get_table()[id][prio] = fcn;
 
     //----------------------------------------------------------------//
-    UHD_LOGV(always) << "register_converter: " << id.to_pp_string() << std::endl
-        << "    prio: " << prio << std::endl
-        << std::endl
-    ;
+    //UHD_LOG_TRACE("CONVERT", boost::format("register_converter: %s prio: %s") % id.to_string() % prio)
     //----------------------------------------------------------------//
 }
 
@@ -100,12 +86,11 @@ convert::function_type convert::get_converter(
 
     //find a matching priority
     priority_type best_prio = -1;
-    BOOST_FOREACH(priority_type prio_i, get_table()[id].keys()){
+    for(priority_type prio_i:  get_table()[id].keys()){
         if (prio_i == prio) {
             //----------------------------------------------------------------//
-            UHD_LOGV(always) << "get_converter: For converter ID: " << id.to_pp_string() << std::endl
-                << "Using prio: " << prio << std::endl
-                << std::endl
+            UHD_LOGGER_DEBUG("CONVERT") << "get_converter: For converter ID: " << id.to_pp_string()
+                                        << " Using prio: " << prio;
             ;
             //----------------------------------------------------------------//
             return get_table()[id][prio];
@@ -118,10 +103,8 @@ convert::function_type convert::get_converter(
         "Cannot find a conversion routine [with prio] for " + id.to_pp_string());
 
     //----------------------------------------------------------------//
-    UHD_LOGV(always) << "get_converter: For converter ID: " << id.to_pp_string() << std::endl
-        << "Using prio: " << best_prio << std::endl
-        << std::endl
-    ;
+    UHD_LOGGER_DEBUG("CONVERT") << "get_converter: For converter ID: " << id.to_pp_string()
+                                << " Using prio: " << best_prio;
     //----------------------------------------------------------------//
 
     //otherwise, return best prio
