@@ -1,18 +1,8 @@
 //
 // Copyright 2011 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include <uhd/exception.hpp>
@@ -37,6 +27,7 @@ make_exception_impl("IndexError",            index_error,             lookup_err
 make_exception_impl("KeyError",              key_error,               lookup_error)
 make_exception_impl("TypeError",             type_error,              exception)
 make_exception_impl("ValueError",            value_error,             exception)
+make_exception_impl("NarrowingError",        narrowing_error,         value_error)
 make_exception_impl("RuntimeError",          runtime_error,           exception)
 make_exception_impl("NotImplementedError",   not_implemented_error,   runtime_error)
 make_exception_impl("EnvironmentError",      environment_error,       exception)
