@@ -1,66 +1,52 @@
 //
-// Copyright 2014-2015 Ettus Research LLC
+// Copyright 2014-2016 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 // This file contains the block control functions for block controller classes.
 // See block_ctrl_base_factory.cpp for discovery and factory functions.
 
-#include "ctrl_iface.hpp"
 #include "nocscript/block_iface.hpp"
-#include <uhd/utils/msg.hpp>
+
 #include <uhd/utils/log.hpp>
 #include <uhd/convert.hpp>
 #include <uhd/rfnoc/block_ctrl_base.hpp>
 #include <uhd/rfnoc/constants.hpp>
+#include <uhdlib/utils/compat_check.hpp>
+#include <uhdlib/rfnoc/ctrl_iface.hpp>
+#include <uhdlib/rfnoc/wb_iface_adapter.hpp>
 #include <boost/format.hpp>
-#include <boost/foreach.hpp>
 #include <boost/bind.hpp>
-
-#define UHD_BLOCK_LOG() UHD_LOGV(never)
-
+#include <chrono>
+#include <thread>
+#include <uhd/utils/safe_call.hpp>
 using namespace uhd;
 using namespace uhd::rfnoc;
 using std::string;
 
-/***********************************************************************
- * Helpers
- **********************************************************************/
-//! Convert register to a peek/poke compatible address
-inline uint32_t _sr_to_addr(uint32_t reg) { return reg * 4; };
-inline uint32_t _sr_to_addr64(uint32_t reg) { return reg * 8; }; // for peek64
-
 /***********************************************************************
  * Structors
  **********************************************************************/
 block_ctrl_base::block_ctrl_base(
         const make_args_t &make_args
 ) : _tree(make_args.tree),
-    _transport_is_big_endian(make_args.is_big_endian),
     _ctrl_ifaces(make_args.ctrl_ifaces),
-    _base_address(make_args.base_address & 0xFFF0)
+    _base_address(make_args.base_address & 0xFFF0),
+    _noc_id(sr_read64(SR_READBACK_REG_ID)),
+    _compat_num(sr_read64(SR_READBACK_COMPAT))
 {
-    UHD_BLOCK_LOG() << "block_ctrl_base()" << std::endl;
-
     /*** Identify this block (NoC-ID, block-ID, and block definition) *******/
     // Read NoC-ID (name is passed in through make_args):
-    uint64_t noc_id = sr_read64(SR_READBACK_REG_ID);
-    _block_def = blockdef::make_from_noc_id(noc_id);
-    if (_block_def) UHD_BLOCK_LOG() <<  "Found valid blockdef" << std::endl;
-    if (not _block_def)
+    _block_def = blockdef::make_from_noc_id(_noc_id);
+    if (not _block_def) {
+        UHD_LOG_DEBUG("RFNOC",
+            "No block definition found, using default block configuration "
+            "for block with NOC ID: " + str(boost::format("0x%08X") % _noc_id)
+        );
         _block_def = blockdef::make_from_noc_id(DEFAULT_NOC_ID);
+    }
     UHD_ASSERT_THROW(_block_def);
     // For the block ID, we start with block count 0 and increase until
     // we get a block ID that's not already registered:
@@ -68,20 +54,48 @@ block_ctrl_base::block_ctrl_base(
     while (_tree->exists("xbar/" + _block_id.get_local())) {
         _block_id++;
     }
-    UHD_BLOCK_LOG()
-        << "NOC ID: " << str(boost::format("0x%016X  ") % noc_id)
-        << "Block ID: " << _block_id << std::endl;
+    UHD_LOG_INFO(unique_id(), str(
+            boost::format("Initializing block control (NOC ID: 0x%016X)")
+            % _noc_id
+    ));
+
+    /*** Check compat number ************************************************/
+    assert_fpga_compat(
+            NOC_SHELL_COMPAT_MAJOR,
+            NOC_SHELL_COMPAT_MINOR,
+            _compat_num,
+            "noc_shell",
+            unique_id(),
+            false /* fail_on_minor_behind */
+    );
 
     /*** Initialize property tree *******************************************/
     _root_path = "xbar/" + _block_id.get_local();
-    _tree->create<uint64_t>(_root_path / "noc_id").set(noc_id);
+    _tree->create<uint64_t>(_root_path / "noc_id").set(_noc_id);
 
     /*** Reset block state *******************************************/
+    // We don't know the state of the data-path of this block before
+    // we initialize. If everything tore down properly, the data-path
+    // should be disconnected and thus idle. Reconfiguration of parameters
+    // like SIDs is safe to do in that scenario.
+    // However, if data is still streaming, block configuration
+    // can potentially lock up noc_shell. So we flush the data-path here.
+
+    // Flush is a block-level operation that can be triggered
+    // from any block port.
+    // Do it once before clearing...
+    if (get_ctrl_ports().size() > 0) {
+        _flush(get_ctrl_ports().front());
+    }
+    // Clear flow control and misc state
     clear();
 
     /*** Configure ports ****************************************************/
     size_t n_valid_input_buffers = 0;
-    BOOST_FOREACH(const size_t ctrl_port, get_ctrl_ports()) {
+    for (const size_t ctrl_port : get_ctrl_ports()) {
+        // Set command times to sensible defaults
+        set_command_tick_rate(1.0, ctrl_port);
+        set_command_time(time_spec_t(0.0), ctrl_port);
         // Set source addresses:
         sr_write(SR_BLOCK_SID, get_address(ctrl_port), ctrl_port);
         // Set sink buffer sizes:
@@ -91,11 +105,15 @@ block_ctrl_base::block_ctrl_base(
         size_t buf_size_bytes = BYTES_PER_LINE * (1 << buf_size_log2); // Bytes == 8 * 2^x
         if (buf_size_bytes > 0) n_valid_input_buffers++;
         _tree->create<size_t>(_root_path / "input_buffer_size" / ctrl_port).set(buf_size_bytes);
+        // Set default destination SIDs
+        // Otherwise, the default is someone else's SID, which we don't want
+        sr_write(SR_RESP_IN_DST_SID, 0xFFFF, ctrl_port);
+        sr_write(SR_RESP_OUT_DST_SID, 0xFFFF, ctrl_port);
     }
 
     /*** Register names *****************************************************/
     blockdef::registers_t sregs = _block_def->get_settings_registers();
-    BOOST_FOREACH(const std::string &reg_name, sregs.keys()) {
+    for(const std::string &reg_name:  sregs.keys()) {
         if (DEFAULT_NAMED_SR.has_key(reg_name)) {
             throw uhd::runtime_error(str(
                     boost::format("Register name %s is already defined!")
@@ -106,7 +124,7 @@ block_ctrl_base::block_ctrl_base(
             .set(sregs.get(reg_name));
     }
     blockdef::registers_t rbacks = _block_def->get_readback_registers();
-    BOOST_FOREACH(const std::string &reg_name, rbacks.keys()) {
+    for(const std::string &reg_name:  rbacks.keys()) {
         _tree->create<size_t>(_root_path / "registers"/ "rb" / reg_name)
             .set(rbacks.get(reg_name));
     }
@@ -115,12 +133,14 @@ block_ctrl_base::block_ctrl_base(
     _init_port_defs("in",  _block_def->get_input_ports());
     _init_port_defs("out", _block_def->get_output_ports());
     // FIXME this warning always fails until the input buffer code above is fixed
-    if (_tree->list(_root_path / "ports/in").size() != n_valid_input_buffers) {
-        UHD_MSG(warning) <<
-            boost::format("[%s] defines %d input buffer sizes, but %d input ports")
-            % get_block_id().get() % n_valid_input_buffers % _tree->list(_root_path / "ports/in").size()
-            << std::endl;
-    }
+    //if (_tree->list(_root_path / "ports/in").size() != n_valid_input_buffers) {
+    //    UHD_LOGGER_WARNING(unique_id()) <<
+    //        boost::format("[%s] defines %d input buffer sizes, but %d input ports")
+    //        % get_block_id().get()
+    //        % n_valid_input_buffers
+    //        % _tree->list(_root_path / "ports/in").size()
+    //    ;
+    //}
 
     /*** Init default block args ********************************************/
     _nocscript_iface = nocscript::block_iface::make(this);
@@ -129,7 +149,18 @@ block_ctrl_base::block_ctrl_base(
 
 block_ctrl_base::~block_ctrl_base()
 {
-    _tree->remove(_root_path);
+    UHD_SAFE_CALL(
+       if (get_ctrl_ports().size() > 0) {
+           // Notify the data-path gatekeeper in noc_shell that we are done
+           // with this block. This operation disconnects the noc_block
+           // data-path from noc_shell which dumps all input and output
+           // packets that are in flight, for now and until the setting is
+           // disabled. This prevents long-running blocks without a tear-down
+           // mechanism to gracefully flush.
+           _start_drain(get_ctrl_ports().front());
+       }
+       _tree->remove(_root_path);
+    )
 }
 
 void block_ctrl_base::_init_port_defs(
@@ -138,14 +169,18 @@ void block_ctrl_base::_init_port_defs(
             const size_t first_port_index
 ) {
     size_t port_index = first_port_index;
-    BOOST_FOREACH(const blockdef::port_t &port_def, ports) {
+    for(const blockdef::port_t &port_def:  ports) {
         fs_path port_path = _root_path / "ports" / direction / port_index;
         if (not _tree->exists(port_path)) {
             _tree->create<blockdef::port_t>(port_path);
         }
-        UHD_RFNOC_BLOCK_TRACE()  << "Adding port definition at " << port_path
-            << boost::format(": type = '%s' pkt_size = '%s' vlen = '%s'") % port_def["type"] % port_def["pkt_size"] % port_def["vlen"]
-            << std::endl;
+        UHD_LOGGER_TRACE(unique_id())
+            << "Adding port definition at " << port_path
+            << boost::format(": type = '%s' pkt_size = '%s' vlen = '%s'")
+                % port_def["type"]
+                % port_def["pkt_size"]
+                % port_def["vlen"]
+        ;
         _tree->access<blockdef::port_t>(port_path).set(port_def);
         port_index++;
     }
@@ -155,12 +190,12 @@ void block_ctrl_base::_init_block_args()
 {
     blockdef::args_t args = _block_def->get_args();
     fs_path arg_path = _root_path / "args";
-    BOOST_FOREACH(const size_t port, get_ctrl_ports()) {
+    for (const size_t port : get_ctrl_ports()) {
         _tree->create<std::string>(arg_path / port);
     }
 
     // First, create all nodes.
-    BOOST_FOREACH(const blockdef::arg_t &arg, args) {
+    for (const auto& arg : args) {
         fs_path arg_type_path = arg_path / arg["port"] / arg["name"] / "type";
         _tree->create<std::string>(arg_type_path).set(arg["type"]);
         fs_path arg_val_path  = arg_path / arg["port"] / arg["name"] / "value";
@@ -174,7 +209,7 @@ void block_ctrl_base::_init_block_args()
     // TODO: Add coercer
 #define _SUBSCRIBE_CHECK_AND_RUN(type, arg_tag, error_message) \
     _tree->access<type>(arg_val_path).add_coerced_subscriber(boost::bind((&nocscript::block_iface::run_and_check), _nocscript_iface, arg[#arg_tag], error_message))
-    BOOST_FOREACH(const blockdef::arg_t &arg, args) {
+    for (const auto& arg : args) {
         fs_path arg_val_path = arg_path / arg["port"] / arg["name"] / "value";
         if (not arg["check"].empty()) {
             if (arg["type"] == "string") { _SUBSCRIBE_CHECK_AND_RUN(string, check, arg["check_message"]); }
@@ -193,12 +228,12 @@ void block_ctrl_base::_init_block_args()
     }
 
     // Finally: Set the values. This will call subscribers, if we have any.
-    BOOST_FOREACH(const blockdef::arg_t &arg, args) {
+    for (const auto& arg : args) {
         fs_path arg_val_path = arg_path / arg["port"] / arg["name"] / "value";
         if (not arg["value"].empty()) {
             if (arg["type"] == "int_vector") { throw uhd::runtime_error("not yet implemented: int_vector"); }
-            else if (arg["type"] == "int") { _tree->access<int>(arg_val_path).set(boost::lexical_cast<int>(arg["value"])); }
-            else if (arg["type"] == "double") { _tree->access<double>(arg_val_path).set(boost::lexical_cast<double>(arg["value"])); }
+            else if (arg["type"] == "int") { _tree->access<int>(arg_val_path).set(std::stoi(arg["value"])); }
+            else if (arg["type"] == "double") { _tree->access<double>(arg_val_path).set(std::stod(arg["value"])); }
             else if (arg["type"] == "string") { _tree->access<string>(arg_val_path).set(arg["value"]); }
             else { UHD_THROW_INVALID_CODE_PATH(); }
         }
@@ -208,17 +243,22 @@ void block_ctrl_base::_init_block_args()
 /***********************************************************************
  * FPGA control & communication
  **********************************************************************/
-wb_iface::sptr block_ctrl_base::get_ctrl_iface(const size_t block_port)
+timed_wb_iface::sptr block_ctrl_base::get_ctrl_iface(const size_t block_port)
 {
-    return _ctrl_ifaces[block_port];
+    return boost::make_shared<wb_iface_adapter>(
+        _ctrl_ifaces[block_port],
+        boost::bind(&block_ctrl_base::get_command_tick_rate, this, block_port),
+        boost::bind(&block_ctrl_base::set_command_time, this, _1, block_port),
+        boost::bind(&block_ctrl_base::get_command_time, this, block_port)
+    );
 }
 
 std::vector<size_t> block_ctrl_base::get_ctrl_ports() const
 {
     std::vector<size_t> ctrl_ports;
     ctrl_ports.reserve(_ctrl_ifaces.size());
-    std::pair<size_t, wb_iface::sptr> it;
-    BOOST_FOREACH(it, _ctrl_ifaces) {
+    std::pair<size_t, ctrl_iface::sptr> it;
+    for(auto it:  _ctrl_ifaces) {
         ctrl_ports.push_back(it.first);
     }
     return ctrl_ports;
@@ -226,13 +266,14 @@ std::vector<size_t> block_ctrl_base::get_ctrl_ports() const
 
 void block_ctrl_base::sr_write(const uint32_t reg, const uint32_t data, const size_t port)
 {
-    //UHD_BLOCK_LOG() << "  ";
-    //UHD_RFNOC_BLOCK_TRACE() << boost::format("sr_write(%d, %08X, %d)") % reg % data % port << std::endl;
     if (not _ctrl_ifaces.count(port)) {
         throw uhd::key_error(str(boost::format("[%s] sr_write(): No such port: %d") % get_block_id().get() % port));
     }
     try {
-        _ctrl_ifaces[port]->poke32(_sr_to_addr(reg), data);
+        _ctrl_ifaces[port]->send_cmd_pkt(
+            reg, data, false,
+            _cmd_timespecs[port].to_ticks(_cmd_tickrates[port])
+        );
     }
     catch(const std::exception &ex) {
         throw uhd::io_error(str(boost::format("[%s] sr_write() failed: %s") % get_block_id().get() % ex.what()));
@@ -253,8 +294,6 @@ void block_ctrl_base::sr_write(const std::string &reg, const uint32_t data, cons
         }
         reg_addr = uint32_t(_tree->access<size_t>(_root_path / "registers" / "sr" / reg).get());
     }
-    UHD_BLOCK_LOG() << "  ";
-    UHD_RFNOC_BLOCK_TRACE() << boost::format("sr_write(%s, %08X) ==> ") % reg % data << std::endl;
     return sr_write(reg_addr, data, port);
 }
 
@@ -264,7 +303,11 @@ uint64_t block_ctrl_base::sr_read64(const settingsbus_reg_t reg, const size_t po
         throw uhd::key_error(str(boost::format("[%s] sr_read64(): No such port: %d") % get_block_id().get() % port));
     }
     try {
-        return _ctrl_ifaces[port]->peek64(_sr_to_addr64(reg));
+        return _ctrl_ifaces[port]->send_cmd_pkt(
+                SR_READBACK, reg,
+                true,
+                _cmd_timespecs[port].to_ticks(_cmd_tickrates[port])
+        );
     }
     catch(const std::exception &ex) {
         throw uhd::io_error(str(boost::format("[%s] sr_read64() failed: %s") % get_block_id().get() % ex.what()));
@@ -277,7 +320,11 @@ uint32_t block_ctrl_base::sr_read32(const settingsbus_reg_t reg, const size_t po
         throw uhd::key_error(str(boost::format("[%s] sr_read32(): No such port: %d") % get_block_id().get() % port));
     }
     try {
-        return _ctrl_ifaces[port]->peek32(_sr_to_addr64(reg));
+        return uint32_t(_ctrl_ifaces[port]->send_cmd_pkt(
+                SR_READBACK, reg,
+                true,
+                _cmd_timespecs[port].to_ticks(_cmd_tickrates[port])
+        ));
     }
     catch(const std::exception &ex) {
         throw uhd::io_error(str(boost::format("[%s] sr_read32() failed: %s") % get_block_id().get() % ex.what()));
@@ -287,6 +334,7 @@ uint32_t block_ctrl_base::sr_read32(const settingsbus_reg_t reg, const size_t po
 uint64_t block_ctrl_base::user_reg_read64(const uint32_t addr, const size_t port)
 {
     try {
+        // TODO: When timed readbacks are used, time the second, but not the first
         // Set readback register address
         sr_write(SR_READBACK_ADDR, addr, port);
         // Read readback register via RFNoC
@@ -332,7 +380,7 @@ uint32_t block_ctrl_base::user_reg_read32(const std::string &reg, const size_t p
         ));
     }
     return user_reg_read32(uint32_t(
-        _tree->access<size_t>(_root_path / "registers" / "sr" / reg).get()
+        _tree->access<size_t>(_root_path / "registers" / "rb" / reg).get()
     ), port);
 }
 
@@ -341,37 +389,20 @@ void block_ctrl_base::set_command_time(
         const size_t port
 ) {
     if (port == ANY_PORT) {
-        BOOST_FOREACH(const size_t specific_port, get_ctrl_ports()) {
+        for(const size_t specific_port:  get_ctrl_ports()) {
             set_command_time(time_spec, specific_port);
         }
         return;
     }
-    boost::shared_ptr<ctrl_iface> iface_sptr =
-        boost::dynamic_pointer_cast<ctrl_iface>(get_ctrl_iface(port));
-    if (not iface_sptr) {
-        throw uhd::assertion_error(str(
-            boost::format("[%s] Cannot set command time on port '%d'")
-            % unique_id() % port
-        ));
-    }
 
-    iface_sptr->set_time(time_spec);
+    _cmd_timespecs[port] = time_spec;
     _set_command_time(time_spec, port);
 }
 
 time_spec_t block_ctrl_base::get_command_time(
         const size_t port
 ) {
-    boost::shared_ptr<ctrl_iface> iface_sptr =
-        boost::dynamic_pointer_cast<ctrl_iface>(get_ctrl_iface(port));
-    if (not iface_sptr) {
-        throw uhd::assertion_error(str(
-            boost::format("[%s] Cannot get command time on port '%d'")
-            % unique_id() % port
-        ));
-    }
-
-    return iface_sptr->get_time();
+    return _cmd_timespecs[port];
 }
 
 void block_ctrl_base::set_command_tick_rate(
@@ -379,44 +410,32 @@ void block_ctrl_base::set_command_tick_rate(
         const size_t port
 ) {
     if (port == ANY_PORT) {
-        BOOST_FOREACH(const size_t specific_port, get_ctrl_ports()) {
+        for(const size_t specific_port:  get_ctrl_ports()) {
             set_command_tick_rate(tick_rate, specific_port);
         }
         return;
     }
-    boost::shared_ptr<ctrl_iface> iface_sptr =
-        boost::dynamic_pointer_cast<ctrl_iface>(get_ctrl_iface(port));
-    if (not iface_sptr) {
-        throw uhd::assertion_error(str(
-            boost::format("[%s] Cannot set command time on port '%d'")
-            % unique_id() % port
-        ));
-    }
 
-    iface_sptr->set_tick_rate(tick_rate);
+    _cmd_tickrates[port] = tick_rate;
+}
+
+double block_ctrl_base::get_command_tick_rate(const size_t port)
+{
+    return _cmd_tickrates[port];
 }
 
 void block_ctrl_base::clear_command_time(const size_t port)
 {
-    boost::shared_ptr<ctrl_iface> iface_sptr =
-        boost::dynamic_pointer_cast<ctrl_iface>(get_ctrl_iface(port));
-    if (not iface_sptr) {
-        throw uhd::assertion_error(str(
-            boost::format("[%s] Cannot set command time on port '%d'")
-            % unique_id() % port
-        ));
-    }
-
-    iface_sptr->set_time(time_spec_t(0.0));
+    _cmd_timespecs[port] = time_spec_t(0.0);
 }
 
 void block_ctrl_base::clear()
 {
-    UHD_RFNOC_BLOCK_TRACE() << "block_ctrl_base::clear() " << std::endl;
+    UHD_LOG_TRACE(unique_id(), "block_ctrl_base::clear()");
     // Call parent...
     node_ctrl_base::clear();
     // ...then child
-    BOOST_FOREACH(const size_t port_index, get_ctrl_ports()) {
+    for (const size_t port_index : get_ctrl_ports()) {
         _clear(port_index);
     }
 }
@@ -431,7 +450,7 @@ uint32_t block_ctrl_base::get_address(size_t block_port) {
  **********************************************************************/
 void block_ctrl_base::set_args(const uhd::device_addr_t &args, const size_t port)
 {
-    BOOST_FOREACH(const std::string &key, args.keys()) {
+    for(const std::string &key:  args.keys()) {
         if (_tree->exists(get_arg_path(key, port))) {
             set_arg(key, args.get(key), port);
         }
@@ -455,10 +474,10 @@ void block_ctrl_base::set_arg(const std::string &key, const std::string &val, co
             _tree->access<std::string>(arg_val_path).set(val);
         }
         else if (type == "int") {
-            _tree->access<int>(arg_val_path).set(boost::lexical_cast<int>(val));
+            _tree->access<int>(arg_val_path).set(std::stoi(val));
         }
         else if (type == "double") {
-            _tree->access<double>(arg_val_path).set(boost::lexical_cast<double>(val));
+            _tree->access<double>(arg_val_path).set(std::stod(val));
         }
         else if (type == "int_vector") {
             throw uhd::runtime_error("not yet implemented: int_vector");
@@ -474,7 +493,7 @@ void block_ctrl_base::set_arg(const std::string &key, const std::string &val, co
 device_addr_t block_ctrl_base::get_args(const size_t port) const
 {
     device_addr_t args;
-    BOOST_FOREACH(const std::string &key, _tree->list(_root_path / "args" / port)) {
+    for(const std::string &key:  _tree->list(_root_path / "args" / port)) {
         args[key] = get_arg(key);
     }
     return args;
@@ -496,10 +515,10 @@ std::string block_ctrl_base::get_arg(const std::string &key, const size_t port)
         return _tree->access<std::string>(arg_val_path).get();
     }
     else if (type == "int") {
-        return boost::lexical_cast<std::string>(_tree->access<int>(arg_val_path).get());
+        return std::to_string(_tree->access<int>(arg_val_path).get());
     }
     else if (type == "double") {
-        return boost::lexical_cast<std::string>(_tree->access<double>(arg_val_path).get());
+        return std::to_string(_tree->access<double>(arg_val_path).get());
     }
     else if (type == "int_vector") {
         throw uhd::runtime_error("not yet implemented: int_vector");
@@ -535,7 +554,6 @@ stream_sig_t block_ctrl_base::_resolve_port_def(const blockdef::port_t &port_def
     } else {
         stream_sig.item_type = port_def["type"];
     }
-    //UHD_RFNOC_BLOCK_TRACE() << "  item type: " << stream_sig.item_type << std::endl;
 
     // Vector length
     if (port_def.is_variable("vlen")) {
@@ -546,7 +564,6 @@ stream_sig_t block_ctrl_base::_resolve_port_def(const blockdef::port_t &port_def
     } else {
         stream_sig.vlen = boost::lexical_cast<size_t>(port_def["vlen"]);
     }
-    //UHD_RFNOC_BLOCK_TRACE() << "  vector length: " << stream_sig.vlen << std::endl;
 
     // Packet size
     if (port_def.is_variable("pkt_size")) {
@@ -568,24 +585,87 @@ stream_sig_t block_ctrl_base::_resolve_port_def(const blockdef::port_t &port_def
     } else {
         stream_sig.packet_size = boost::lexical_cast<size_t>(port_def["pkt_size"]);
     }
-    //UHD_RFNOC_BLOCK_TRACE() << "  packet size: " << stream_sig.vlen << std::endl;
 
     return stream_sig;
 }
 
+void block_ctrl_base::_start_drain(const size_t port)
+{
+    // Begin flushing data out of the block by writing to the flushing
+    // registers, then disabling flow control. We do this because we don't know
+    // what state the flow-control module was left in in the previous run
+    sr_write(SR_CLEAR_TX_FC, 0x2, port);
+    sr_write(SR_CLEAR_RX_FC, 0x2, port);
+    sr_write(SR_FLOW_CTRL_WINDOW_EN, 0, port);
+}
+
+bool block_ctrl_base::_flush(const size_t port)
+{
+    UHD_LOG_TRACE(unique_id(), "block_ctrl_base::_flush (port=" << port << ")");
+
+    auto is_data_streaming = [this](int time_ms) -> bool {
+        // noc_shell has 2 16-bit counters (one for TX and one for RX) in the top
+        // 32 bits of the SR_READBACK_REG_GLOBAL_PARAMS. For all the checks below
+        // we want to make sure that the counts are not changing i.e. no data is
+        // streaming. So we just look at the two counters together as a single
+        // 32-bit quantity.
+        auto old_cnts = static_cast<uint32_t>(this->sr_read64(SR_READBACK_REG_GLOBAL_PARAMS) >> 32);
+        std::this_thread::sleep_for(std::chrono::milliseconds(time_ms));
+        auto new_cnts = static_cast<uint32_t>(this->sr_read64(SR_READBACK_REG_GLOBAL_PARAMS) >> 32);
+        return (new_cnts != old_cnts);
+    };
+
+    // We always want to try flushing out data. This is done by starting to
+    // drain the data out of the block, then checking if counts have changed.
+    // If a change is detected, this is most likely because the last
+    // session terminated abnormally or if logic in a noc_block is
+    // misbehaving. This is a situation that we may not be able to
+    // recover from because we are in a partially initialized state.
+    // We will try to at least not lock up the FPGA.
+
+    // Disconnect the RX and TX data paths and let them flush.
+    // A timeout of 2s is chosen to be conservative. It needs to account for:
+    // - Upstream blocks that weren't terminated to run out of FC credits
+    // - This block which might be finishing up with its data output
+    constexpr int FLUSH_TIMEOUT_MS = 2000;  // This is approximate
+    bool success = false;
+    _start_drain(port);
+    for (int i = 0; i < FLUSH_TIMEOUT_MS/10; i++) {
+        if (not is_data_streaming(10)) {
+            success = true;
+            break;
+        }
+    }
+    // Stop flushing
+    sr_write(SR_CLEAR_TX_FC, 0x0, port);    // Enable TX data-path
+    sr_write(SR_CLEAR_RX_FC, 0x0, port);    // Enable RX data-path
+
+    if (not success) {
+        // Print a warning only if data was still flushing
+        // after the timeout elapsed
+        UHD_LOGGER_WARNING(unique_id()) <<
+            "This block seems to be busy most likely due to the abnormal termination of a previous "
+            "session. Attempted recovery but it may not have worked depending on the behavior of "
+            "other blocks in the design. Please restart the application.";
+    }
+    return success;
+}
+
 
 /***********************************************************************
  * Hooks & Derivables
  **********************************************************************/
 void block_ctrl_base::_clear(const size_t port)
 {
-    UHD_RFNOC_BLOCK_TRACE() << "block_ctrl_base::_clear() " << std::endl;
-    sr_write(SR_CLEAR_TX_FC, 0x00C1EA12, port); // 'CLEAR', but we can write anything, really
-    sr_write(SR_CLEAR_RX_FC, 0x00C1EA12, port); // 'CLEAR', but we can write anything, really
+    UHD_LOG_TRACE(unique_id(), "block_ctrl_base::_clear()");
+    sr_write(SR_CLEAR_TX_FC, 0x1, port);    // Write 1 to trigger a single cycle clear event
+    sr_write(SR_CLEAR_TX_FC, 0x0, port);    // Write 0 to reset the clear flag
+    sr_write(SR_CLEAR_RX_FC, 0x1, port);    // Write 1 to trigger a single cycle clear event
+    sr_write(SR_CLEAR_RX_FC, 0x0, port);    // Write 0 to reset the clear flag
 }
 
 void block_ctrl_base::_set_command_time(const time_spec_t & /*time_spec*/, const size_t /*port*/)
 {
-    UHD_RFNOC_BLOCK_TRACE() << "block_ctrl_base::_set_command_time() ";
+    UHD_LOG_TRACE(unique_id(), "block_ctrl_base::_set_command_time()");
 }
 // vim: sw=4 et:
