@@ -1,27 +1,16 @@
 //
 // Copyright 2010-2013 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "libusb1_base.hpp"
 #include <uhd/transport/usb_zero_copy.hpp>
 #include <uhd/transport/buffer_pool.hpp>
 #include <uhd/transport/bounded_buffer.hpp>
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <uhd/exception.hpp>
-#include <boost/foreach.hpp>
 #include <boost/format.hpp>
 #include <boost/function.hpp>
 #include <boost/bind.hpp>
@@ -274,19 +263,19 @@ public:
     ~libusb_zero_copy_single(void)
     {
         //cancel all transfers
-        BOOST_FOREACH(libusb_transfer *lut, _all_luts)
+        for(libusb_transfer *lut:  _all_luts)
         {
             libusb_cancel_transfer(lut);
         }
 
         //process all transfers until timeout occurs
-        BOOST_FOREACH(libusb_zero_copy_mb *mb, _enqueued)
+        for(libusb_zero_copy_mb *mb:  _enqueued)
         {
             mb->wait_for_completion(0.01);
         }
 
         //free all transfers
-        BOOST_FOREACH(libusb_transfer *lut, _all_luts)
+        for(libusb_transfer *lut:  _all_luts)
         {
             libusb_free_transfer(lut);
         }
