@@ -1,39 +1,27 @@
 //
 // Copyright 2011-2013 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifndef INCLUDED_LIBUHD_TRANSPORT_SUPER_RECV_PACKET_HANDLER_HPP
 #define INCLUDED_LIBUHD_TRANSPORT_SUPER_RECV_PACKET_HANDLER_HPP
 
-#include "../rfnoc/rx_stream_terminator.hpp"
 #include <uhd/config.hpp>
 #include <uhd/exception.hpp>
 #include <uhd/convert.hpp>
 #include <uhd/stream.hpp>
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <uhd/utils/tasks.hpp>
 #include <uhd/utils/byteswap.hpp>
 #include <uhd/types/metadata.hpp>
 #include <uhd/transport/vrt_if_packet.hpp>
 #include <uhd/transport/zero_copy.hpp>
+#include <uhdlib/rfnoc/rx_stream_terminator.hpp>
 #include <boost/dynamic_bitset.hpp>
-#include <boost/foreach.hpp>
 #include <boost/function.hpp>
 #include <boost/format.hpp>
-#include <boost/bind.hpp>
 #include <boost/make_shared.hpp>
 #include <iostream>
 #include <vector>
@@ -279,6 +267,8 @@ public:
                 timeout, accum_num_samps*_bytes_per_cpu_item
             );
 
+            metadata.end_of_burst = _queue_metadata.end_of_burst;
+
             //metadata had an error code set, store for next call and return
             if (_queue_metadata.error_code != rx_metadata_t::ERROR_CODE_NONE){
                 _queue_error_for_next_call = true;
@@ -333,14 +323,14 @@ private:
         void reset()
         {
             buff.reset();
-            vrt_hdr = NULL;
-            time = time_spec_t(0.0);
-            copy_buff = NULL;
+            vrt_hdr = nullptr;
+            time = 0;
+            copy_buff = nullptr;
         }
         managed_recv_buffer::sptr buff;
         const uint32_t *vrt_hdr;
         vrt::if_packet_info_t ifpi;
-        time_spec_t time;
+        uint64_t time;
         const char *copy_buff;
     };
 
@@ -349,6 +339,7 @@ private:
         buffers_info_type(const size_t size):
             std::vector<per_buffer_info_type>(size),
             indexes_todo(size, true),
+            alignment_time(0),
             alignment_time_valid(false),
             data_bytes_to_copy(0),
             fragment_offset_in_samps(0)
@@ -356,7 +347,7 @@ private:
         void reset()
         {
             indexes_todo.set();
-            alignment_time = time_spec_t(0.0);
+            alignment_time = 0;
             alignment_time_valid = false;
             data_bytes_to_copy = 0;
             fragment_offset_in_samps = 0;
@@ -365,7 +356,7 @@ private:
                 at(i).reset();
         }
         boost::dynamic_bitset<> indexes_todo; //used in alignment logic
-        time_spec_t alignment_time; //used in alignment logic
+        uint64_t alignment_time; //used in alignment logic
         bool alignment_time_valid; //used in alignment logic
         size_t data_bytes_to_copy; //keeps track of state
         size_t fragment_offset_in_samps; //keeps track of state
@@ -410,7 +401,7 @@ private:
         //get a single packet from the transport layer
         managed_recv_buffer::sptr &buff = curr_buffer_info.buff;
         buff = _props[index].get_buff(timeout);
-        if (buff.get() == NULL) return PACKET_TIMEOUT_ERROR;
+        if (buff.get() == nullptr) return PACKET_TIMEOUT_ERROR;
 
         #ifdef  ERROR_INJECT_DROPPED_PACKETS
         if (++recvd_packets > 1000)
@@ -418,7 +409,7 @@ private:
             recvd_packets = 0;
             buff.reset();
             buff = _props[index].get_buff(timeout);
-            if (buff.get() == NULL) return PACKET_TIMEOUT_ERROR;
+            if (buff.get() == nullptr) return PACKET_TIMEOUT_ERROR;
         }
         #endif
 
@@ -433,7 +424,7 @@ private:
         info.ifpi.num_packet_words32 = num_packet_words32 - _header_offset_words32;
         info.vrt_hdr = buff->cast<const uint32_t *>() + _header_offset_words32;
         _vrt_unpacker(info.vrt_hdr, info.ifpi);
-        info.time = time_spec_t::from_ticks(info.ifpi.tsf, _tick_rate); //assumes has_tsf is true
+        info.time = info.ifpi.tsf; //assumes has_tsf is true
         info.copy_buff = reinterpret_cast<const char *>(info.vrt_hdr + info.ifpi.num_header_words32);
 
         //handle flow control
@@ -461,7 +452,7 @@ private:
         const size_t expected_packet_count = _props[index].packet_count;
         _props[index].packet_count = (info.ifpi.packet_count + 1) & seq_mask;
         if (expected_packet_count != info.ifpi.packet_count){
-            //UHD_MSG(status) << "expected: " << expected_packet_count << " got: " << info.ifpi.packet_count << std::endl;
+            //UHD_LOGGER_INFO("STREAMER") << "expected: " << expected_packet_count << " got: " << info.ifpi.packet_count;
             if (_props[index].handle_flowctrl) {
                 // Always update flow control in this case, because we don't
                 // know which packet was dropped and what state the upstream
@@ -490,6 +481,8 @@ private:
         for (size_t i = 0; i < _props.size(); i++)
         {
             per_buffer_info_type prev_buffer_info, curr_buffer_info;
+            prev_buffer_info.reset();
+            curr_buffer_info.reset();
             while (true)
             {
                 //receive a single packet from the transport
@@ -503,6 +496,7 @@ private:
                             curr_buffer_info,
                             timeout) == PACKET_TIMEOUT_ERROR) break;
                 } catch(...){}
+                curr_buffer_info.buff.reset();  // Let my buffer go!
                 prev_buffer_info = curr_buffer_info;
                 curr_buffer_info.reset();
             }
@@ -524,13 +518,33 @@ private:
             info.alignment_time = info[index].time;
             info.indexes_todo.set();
             info.indexes_todo.reset(index);
+            // release the other buffers
+            for (size_t i = 0; i < info.size(); i++)
+            {
+                if (i != index)
+                {
+                    info[i].reset();
+                }
+            }
             info.data_bytes_to_copy = info[index].ifpi.num_payload_bytes;
+            // reset start_of_burst and end_of_burst states
+            info.metadata.start_of_burst = info[index].ifpi.sob;
+            info.metadata.end_of_burst = info[index].ifpi.eob;
         }
 
         //if the sequence id matches:
         //  remove this index from the list and continue
         else if (info[index].time == info.alignment_time){
             info.indexes_todo.reset(index);
+            // All channels should have sob set at the same time, so only
+            // set start_of burst if all channels have sob set.
+            info.metadata.start_of_burst &= info[index].ifpi.sob;
+            // If any channel indicates eob, no more data will be received for
+            // that channel so set end_of_burst for any eob.
+            info.metadata.end_of_burst |= info[index].ifpi.eob;
+        } else {
+            // Not going to use this buffer, so release it
+            info[index].reset();
         }
 
         //if the sequence id is older:
@@ -554,6 +568,8 @@ private:
         buffers_info_type &curr_info = get_curr_buffer_info();
         buffers_info_type &next_info = get_next_buffer_info();
 
+        curr_info.metadata.error_code = rx_metadata_t::ERROR_CODE_NONE;
+
         //Loop until we get a message of an aligned set of buffers:
         // - Receive a single packet and extract its info.
         // - Handle the packet type yielded by the receive.
@@ -574,9 +590,9 @@ private:
 
             //handle the case where a bad header exists
             catch(const uhd::value_error &e){
-                UHD_MSG(error) << boost::format(
+                UHD_LOGGER_ERROR("STREAMER") << boost::format(
                     "The receive packet handler caught a value exception.\n%s"
-                ) % e.what() << std::endl;
+                    ) % e.what();
                 std::swap(curr_info, next_info); //save progress from curr -> next
                 curr_info.metadata.error_code = rx_metadata_t::ERROR_CODE_BAD_PACKET;
                 return;
@@ -599,9 +615,11 @@ private:
                 break;
 
             case PACKET_INLINE_MESSAGE:
+                curr_info[index].buff.reset();  // No data, so release the buffer
+                curr_info[index].copy_buff = nullptr;
                 std::swap(curr_info, next_info); //save progress from curr -> next
                 curr_info.metadata.has_time_spec = next_info[index].ifpi.has_tsf;
-                curr_info.metadata.time_spec = next_info[index].time;
+                curr_info.metadata.time_spec = time_spec_t::from_ticks(next_info[index].time, _tick_rate);
                 curr_info.metadata.error_code = rx_metadata_t::error_code_t(get_context_code(next_info[index].vrt_hdr, next_info[index].ifpi));
                 if (curr_info.metadata.error_code == rx_metadata_t::ERROR_CODE_OVERFLOW){
                     // Not sending flow control would cause timeouts due to source flow control locking up.
@@ -614,10 +632,8 @@ private:
                     rx_metadata_t metadata = curr_info.metadata;
                     _props[index].handle_overflow();
                     curr_info.metadata = metadata;
-                    UHD_MSG(fastpath) << "O";
+                    UHD_LOG_FASTPATH("O");
                 }
-                curr_info[index].buff.reset();
-                curr_info[index].copy_buff = NULL;
                 return;
 
             case PACKET_TIMEOUT_ERROR:
@@ -636,14 +652,14 @@ private:
                     prev_info[index].ifpi.num_payload_words32*sizeof(uint32_t)/_bytes_per_otw_item, _samp_rate);
                 curr_info.metadata.out_of_sequence = true;
                 curr_info.metadata.error_code = rx_metadata_t::ERROR_CODE_OVERFLOW;
-                UHD_MSG(fastpath) << "D";
+                UHD_LOG_FASTPATH("D");
                 return;
 
             }
 
             //too many iterations: detect alignment failure
             if (iterations++ > _alignment_failure_threshold){
-                UHD_MSG(error) << boost::format(
+                UHD_LOGGER_ERROR("STREAMER") << boost::format(
                     "The receive packet handler failed to time-align packets.\n"
                     "%u received packets were processed by the handler.\n"
                     "However, a timestamp match could not be determined.\n"
@@ -658,17 +674,15 @@ private:
 
         //set the metadata from the buffer information at index zero
         curr_info.metadata.has_time_spec = curr_info[0].ifpi.has_tsf;
-        curr_info.metadata.time_spec = curr_info[0].time;
+        curr_info.metadata.time_spec = time_spec_t::from_ticks(curr_info[0].time, _tick_rate);
         curr_info.metadata.more_fragments = false;
         curr_info.metadata.fragment_offset = 0;
-        curr_info.metadata.start_of_burst = curr_info[0].ifpi.sob;
-        curr_info.metadata.end_of_burst = curr_info[0].ifpi.eob;
         curr_info.metadata.error_code = rx_metadata_t::ERROR_CODE_NONE;
 
     }
 
     /*******************************************************************
-     * Receive a single packet:
+     * Receive a single packet on all channels
      * Handles fragmentation, messages, errors, and copy-conversion.
      * When no fragments are available, call the get aligned buffers.
      * Then copy-convert available data into the user's IO buffers.
