@@ -1,18 +1,8 @@
 //
 // Copyright 2010-2011,2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "udp_common.hpp"
@@ -31,11 +21,12 @@ public:
     udp_simple_impl(
         const std::string &addr, const std::string &port, bool bcast, bool connect
     ):_connected(connect){
-        UHD_LOG << boost::format("Creating udp transport for %s %s") % addr % port << std::endl;
+        UHD_LOGGER_TRACE("UDP") << boost::format("Creating udp transport for %s %s") % addr % port ;
 
         //resolve the address
         asio::ip::udp::resolver resolver(_io_service);
-        asio::ip::udp::resolver::query query(asio::ip::udp::v4(), addr, port);
+        asio::ip::udp::resolver::query query(asio::ip::udp::v4(), addr, port,
+            asio::ip::resolver_query_base::all_matching);
         _send_endpoint = *resolver.resolve(query);
 
         //create and open the socket
@@ -64,6 +55,10 @@ public:
         return _recv_endpoint.address().to_string();
     }
 
+    std::string get_send_addr(void){
+        return _send_endpoint.address().to_string();
+    }
+
 private:
     bool                    _connected;
     asio::io_service        _io_service;
