@@ -1,18 +1,8 @@
 //
 // Copyright 2010-2011 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include <uhd/transport/usb_device_handle.hpp>
@@ -27,6 +17,14 @@ usb_control::~usb_control(void){
     /* NOP */
 }
 
+usb_device_handle::~usb_device_handle(void) {
+    /* NOP */
+}
+
+usb_zero_copy::~usb_zero_copy(void) {
+    /* NOP */
+}
+
 std::vector<usb_device_handle::sptr> usb_device_handle::get_device_list(uint16_t, uint16_t){
     return std::vector<usb_device_handle::sptr>(); //empty list
 }
