@@ -1,18 +1,8 @@
 //
 // Copyright 2011,2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "codec_ctrl.hpp"
@@ -259,7 +249,7 @@ void b100_codec_ctrl_impl::write_aux_dac(aux_dac_t which, double volts){
  **********************************************************************/
 void b100_codec_ctrl_impl::send_reg(uint8_t addr){
     uint32_t reg = _ad9862_regs.get_write_reg(addr);
-    UHD_LOGV(rarely) << "codec control write reg: " << std::hex << reg << std::endl;
+    UHD_LOGGER_TRACE("B100") << "codec control write reg: " << std::hex << reg ;
     _iface->transact_spi(
         B100_SPI_SS_AD9862,
         spi_config_t::EDGE_RISE,
@@ -269,13 +259,13 @@ void b100_codec_ctrl_impl::send_reg(uint8_t addr){
 
 void b100_codec_ctrl_impl::recv_reg(uint8_t addr){
     uint32_t reg = _ad9862_regs.get_read_reg(addr);
-    UHD_LOGV(rarely) << "codec control read reg: " << std::hex << reg << std::endl;
+    UHD_LOGGER_TRACE("B100") << "codec control read reg: " << std::hex << reg ;
     uint32_t ret = _iface->transact_spi(
         B100_SPI_SS_AD9862,
         spi_config_t::EDGE_RISE,
         reg, 16, true /*rb*/
     );
-    UHD_LOGV(rarely) << "codec control read ret: " << std::hex << uint16_t(ret & 0xFF) << std::endl;
+    UHD_LOGGER_TRACE("B100") << "codec control read ret: " << std::hex << uint16_t(ret & 0xFF) ;
     _ad9862_regs.set_reg(addr, uint8_t(ret&0xff));
 }
 
