@@ -1,38 +1,29 @@
 //
 // Copyright 2012-2013 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "b200_iface.hpp"
 
-#include "../../utils/ihex.hpp"
 #include <uhd/config.hpp>
-#include <uhd/utils/msg.hpp>
 #include <uhd/utils/log.hpp>
 #include <uhd/exception.hpp>
+#include <uhdlib/utils/ihex.hpp>
+
 #include <boost/functional/hash.hpp>
-#include <boost/thread/thread.hpp>
-#include <stdint.h>
 #include <boost/lexical_cast.hpp>
 #include <boost/format.hpp>
+#include <libusb.h>
 #include <fstream>
 #include <string>
 #include <vector>
 #include <cstring>
 #include <iomanip>
-#include <libusb.h>
+#include <chrono>
+#include <thread>
+#include <stdint.h>
 
 //! libusb_error_name is only in newer API
 #ifndef HAVE_LIBUSB_ERROR_NAME
@@ -211,9 +202,10 @@ public:
 
     void load_firmware(const std::string filestring, UHD_UNUSED(bool force) = false)
     {
-        if (load_img_msg)
-            UHD_MSG(status) << "Loading firmware image: "
-                            << filestring << "..." << std::flush;
+        if (load_img_msg) {
+            UHD_LOGGER_INFO("B200") << "Loading firmware image: "
+                            << filestring << "...";
+        }
 
         ihex_reader file_reader(filestring);
         try {
@@ -227,7 +219,6 @@ public:
             throw uhd::io_error(str(boost::format("Could not load firmware: \n%s") % e.what()));
         }
 
-        UHD_MSG(status) << std::endl;
 
         //TODO
         //usrp_set_firmware_hash(hash); //set hash before reset
@@ -235,7 +226,7 @@ public:
         /* Success! Let the system settle. */
         // TODO: Replace this with a polling loop in the FX3, or find out
         // what the actual, correct timeout value is.
-        boost::this_thread::sleep(boost::posix_time::milliseconds(1000));
+        std::this_thread::sleep_for(std::chrono::milliseconds(1000));
     }
 
     void reset_fx3(void) {
@@ -441,13 +432,15 @@ public:
                 return fx3_state;
             }
 
-            boost::this_thread::sleep(boost::posix_time::milliseconds(10));
+            std::this_thread::sleep_for(std::chrono::milliseconds(10));
 
             wait_count++;
         } while(fx3_state != FX3_STATE_FPGA_READY);
 
-        if (load_img_msg) UHD_MSG(status) << "Loading FPGA image: " \
-            << filestring << "..." << std::flush;
+        if (load_img_msg) {
+            UHD_LOGGER_INFO("B200") << "Loading FPGA image: "
+                                    << filestring << "...";
+        }
 
         bytes_to_xfer = 1;
         ret = fx3_control_write(B200_VREQ_FPGA_START, 0, 0, out_buff, bytes_to_xfer, 1000);
@@ -464,7 +457,7 @@ public:
                 return fx3_state;
             }
 
-            boost::this_thread::sleep(boost::posix_time::milliseconds(10));
+            std::this_thread::sleep_for(std::chrono::milliseconds(10));
 
             wait_count++;
         } while(fx3_state != FX3_STATE_CONFIGURING_FPGA);
@@ -485,15 +478,20 @@ public:
             else if (nwritten != transfer_count)
                 throw uhd::io_error((boost::format("load_fpga: short write while transferring bitstream to FX3  (expecting: %d, returned: %d)") % transfer_count % nwritten).str());
 
+            const size_t LOG_GRANULARITY = 10; // %. Keep this an integer divisor of 100.
             if (load_img_msg)
             {
-                if (bytes_sent == 0) UHD_MSG(status) << "  0%" << std::flush;
-                const size_t percent_before = size_t((bytes_sent*100)/file_size);
+                if (bytes_sent == 0) UHD_LOGGER_DEBUG("B200") << "  0%" << std::flush;
+                const size_t percent_before =
+                    size_t((bytes_sent*100)/file_size) -
+                    (size_t((bytes_sent*100)/file_size) % LOG_GRANULARITY);
                 bytes_sent += transfer_count;
-                const size_t percent_after = size_t((bytes_sent*100)/file_size);
+                const size_t percent_after =
+                    size_t((bytes_sent*100)/file_size) -
+                    (size_t((bytes_sent*100)/file_size) % LOG_GRANULARITY);
                 if (percent_before != percent_after)
                 {
-                    UHD_MSG(status) << "\b\b\b\b" << std::setw(3) << percent_after << "%" << std::flush;
+                    UHD_LOGGER_DEBUG("B200") << std::setw(3) << percent_after << "%";
                 }
             }
         }
@@ -508,15 +506,16 @@ public:
                 return fx3_state;
             }
 
-            boost::this_thread::sleep(boost::posix_time::milliseconds(10));
+            std::this_thread::sleep_for(std::chrono::milliseconds(10));
 
             wait_count++;
         } while(fx3_state != FX3_STATE_RUNNING);
 
         usrp_set_fpga_hash(hash);
 
-        if (load_img_msg)
-            UHD_MSG(status) << "\b\b\b\b done" << std::endl;
+        if (load_img_msg) {
+            UHD_LOGGER_DEBUG("B200") << "FPGA image loaded!";
+        }
 
         return 0;
     }
