@@ -1,26 +1,16 @@
 //
 // Copyright 2012-2015 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "b200_regs.hpp"
 #include "b200_impl.hpp"
-#include "validate_subdev_spec.hpp"
+#include <uhdlib/usrp/common/validate_subdev_spec.hpp>
+#include <uhdlib/usrp/common/async_packet_handler.hpp>
 #include "../../transport/super_recv_packet_handler.hpp"
 #include "../../transport/super_send_packet_handler.hpp"
-#include "async_packet_handler.hpp"
 #include <uhd/utils/math.hpp>
 #include <boost/bind.hpp>
 #include <boost/make_shared.hpp>
@@ -45,7 +35,7 @@ void b200_impl::check_tick_rate_with_current_streamers(double rate)
 size_t b200_impl::max_chan_count(const std::string &direction /* = "" */)
 {
     size_t max_count = 0;
-    BOOST_FOREACH(radio_perifs_t &perif, _radio_perifs)
+    for(radio_perifs_t &perif:  _radio_perifs)
     {
         if ((direction == "RX" or direction.empty()) and not perif.rx_streamer.expired()) {
             boost::shared_ptr<sph::recv_packet_streamer> rx_streamer =
@@ -97,7 +87,7 @@ void b200_impl::set_auto_tick_rate(
     for (int i = 0; i < 2; i++) { // Loop through rx and tx
         std::string dir = (i == 0) ? "tx" : "rx";
         // We assume all 'set' DSPs are being used.
-        BOOST_FOREACH(const std::string &dsp_no, _tree->list(str(boost::format("/mboards/0/%s_dsps") % dir))) {
+        for(const std::string &dsp_no:  _tree->list(str(boost::format("/mboards/0/%s_dsps") % dir))) {
             fs_path dsp_path = str(boost::format("/mboards/0/%s_dsps/%s") % dir % dsp_no);
             if (dsp_path == tree_dsp_path) {
                 continue;
@@ -135,9 +125,9 @@ void b200_impl::set_auto_tick_rate(
             _tree->access<double>("/mboards/0/tick_rate").set(new_rate);
         }
     } catch (const uhd::value_error &) {
-        UHD_MSG(warning)
-            << "Cannot automatically determine an appropriate tick rate for these sampling rates." << std::endl
-            << "Consider using different sampling rates, or manually specify a suitable master clock rate." << std::endl;
+        UHD_LOGGER_WARNING("B200")
+            << "Cannot automatically determine an appropriate tick rate for these sampling rates." 
+            << "Consider using different sampling rates, or manually specify a suitable master clock rate." ;
         return; // Let the others handle this
     }
 }
@@ -146,14 +136,14 @@ void b200_impl::update_tick_rate(const double new_tick_rate)
 {
     check_tick_rate_with_current_streamers(new_tick_rate);
 
-    BOOST_FOREACH(radio_perifs_t &perif, _radio_perifs)
+    for(radio_perifs_t &perif:  _radio_perifs)
     {
         boost::shared_ptr<sph::recv_packet_streamer> my_streamer =
             boost::dynamic_pointer_cast<sph::recv_packet_streamer>(perif.rx_streamer.lock());
         if (my_streamer) my_streamer->set_tick_rate(new_tick_rate);
         perif.framer->set_tick_rate(new_tick_rate);
     }
-    BOOST_FOREACH(radio_perifs_t &perif, _radio_perifs)
+    for(radio_perifs_t &perif:  _radio_perifs)
     {
         boost::shared_ptr<sph::send_packet_streamer> my_streamer =
             boost::dynamic_pointer_cast<sph::send_packet_streamer>(perif.tx_streamer.lock());
@@ -356,7 +346,7 @@ boost::optional<uhd::msg_task::msg_type_t> b200_impl::handle_async_task(
         }
         catch(const std::exception &ex)
         {
-            UHD_MSG(error) << "Error parsing ctrl packet: " << ex.what() << std::endl;
+            UHD_LOGGER_ERROR("B200") << "Error parsing ctrl packet: " << ex.what();
             break;
         }
 
@@ -370,7 +360,7 @@ boost::optional<uhd::msg_task::msg_type_t> b200_impl::handle_async_task(
 
     //doh!
     default:
-        UHD_MSG(error) << "Got a ctrl packet with unknown SID " << sid << std::endl;
+        UHD_LOGGER_ERROR("B200") << "Got a ctrl packet with unknown SID " << sid;
     }
     return boost::none;
 }
