@@ -1,28 +1,18 @@
 //
 // Copyright 2012,2015 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#include "fifo_ctrl_excelsior.hpp"
-#include "async_packet_handler.hpp"
+#include <uhdlib/usrp/common/fifo_ctrl_excelsior.hpp>
+#include <uhdlib/usrp/common/async_packet_handler.hpp>
 #include <uhd/exception.hpp>
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <uhd/utils/byteswap.hpp>
 #include <uhd/utils/tasks.hpp>
 #include <uhd/utils/safe_call.hpp>
-#include <uhd/utils/thread_priority.hpp>
+#include <uhd/utils/thread.hpp>
 #include <uhd/transport/vrt_if_packet.hpp>
 #include <uhd/transport/bounded_buffer.hpp>
 #include <boost/thread/mutex.hpp>
@@ -97,7 +87,7 @@ public:
             vrt::if_hdr_unpack_le(pkt, packet_info);
         }
         catch(const std::exception &ex){
-            UHD_MSG(error) << "FIFO ctrl bad VITA packet: " << ex.what() << std::endl;
+            UHD_LOGGER_ERROR("UHD") << "FIFO ctrl bad VITA packet: " << ex.what();
         }
         if (packet_info.has_sid and packet_info.sid == _config.ctrl_sid_base){
             ctrl_result_t res = ctrl_result_t();
@@ -112,7 +102,7 @@ public:
             standard_async_msg_prints(metadata);
         }
         else{
-            UHD_MSG(error) << "FIFO ctrl got unknown SID: " << packet_info.sid << std::endl;
+            UHD_LOGGER_ERROR("UHD") << "FIFO ctrl got unknown SID: " << packet_info.sid ;
         }
     }
 
