@@ -6,6 +6,7 @@
 //
 
 #include <uhd/types/dict.hpp>
+#include <uhd/types/ranges.hpp>
 #include <uhdlib/usrp/cores/rx_frontend_core_3000.hpp>
 #include <uhdlib/usrp/cores/dsp_core_utils.hpp>
 #include <boost/math/special_functions/round.hpp>
@@ -34,6 +35,11 @@ using namespace uhd;
 #define OFFSET_SET   (1ul << 30)
 #define FLAG_MASK (OFFSET_FIXED | OFFSET_SET)
 
+namespace {
+    static const double DC_OFFSET_MIN = -1.0;
+    static const double DC_OFFSET_MAX = 1.0;
+}
+
 using namespace uhd::usrp;
 
 static uint32_t fs_to_bits(const double num, const size_t bits){
@@ -75,6 +81,8 @@ public:
         uint32_t mapping_reg_val = 0;
         switch (fe_conn.get_sampling_mode()) {
         case fe_connection_t::REAL:
+            mapping_reg_val = FLAG_DSP_RX_MAPPING_REAL_MODE;
+            break;
         case fe_connection_t::HETERODYNE:
             mapping_reg_val = FLAG_DSP_RX_MAPPING_REAL_MODE|FLAG_DSP_RX_MAPPING_REAL_DECIM;
             break;
@@ -138,6 +146,9 @@ public:
     }
 
     void populate_subtree(uhd::property_tree::sptr subtree) {
+        subtree->create<uhd::meta_range_t>("dc_offset/range")
+                .set(meta_range_t(DC_OFFSET_MIN, DC_OFFSET_MAX))
+        ;
         subtree->create<std::complex<double> >("dc_offset/value")
             .set(DEFAULT_DC_OFFSET_VALUE)
             .set_coercer(boost::bind(&rx_frontend_core_3000::set_dc_offset, this, _1))
