@@ -1,26 +1,16 @@
 //
 // Copyright 2011-2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
-#include "tx_dsp_core_3000.hpp"
-#include "dsp_core_utils.hpp"
+#include <uhdlib/usrp/cores/tx_dsp_core_3000.hpp>
+#include <uhdlib/usrp/cores/dsp_core_utils.hpp>
 #include <uhd/types/dict.hpp>
 #include <uhd/exception.hpp>
 #include <uhd/utils/math.hpp>
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <boost/assign/list_of.hpp>
 #include <boost/math/special_functions/round.hpp>
 #include <boost/thread/thread.hpp> //sleep
@@ -63,6 +53,7 @@ public:
 
     void set_tick_rate(const double rate){
         _tick_rate = rate;
+        set_freq(_current_freq);
     }
 
     void set_link_rate(const double rate){
@@ -103,7 +94,7 @@ public:
 
         if (interp > 1 and hb0 == 0 and hb1 == 0)
         {
-            UHD_MSG(warning) << boost::format(
+            UHD_LOGGER_WARNING("CORES") << boost::format(
                 "The requested interpolation is odd; the user should expect CIC rolloff.\n"
                 "Select an even interpolation to ensure that a halfband filter is enabled.\n"
                 "interpolation = dsp_rate/samp_rate -> %d = (%f MHz)/(%f MHz)\n"
@@ -141,9 +132,14 @@ public:
         int32_t freq_word;
         get_freq_and_freq_word(requested_freq, _tick_rate, actual_freq, freq_word);
         _iface->poke32(REG_DSP_TX_FREQ, uint32_t(freq_word));
+        _current_freq = actual_freq;
         return actual_freq;
     }
 
+    double get_freq(void){
+        return _current_freq;
+    }
+
     uhd::meta_range_t get_freq_range(void){
         return uhd::meta_range_t(-_tick_rate/2, +_tick_rate/2, _tick_rate/std::pow(2.0, 32));
     }
@@ -189,6 +185,7 @@ public:
         subtree->create<double>("freq/value")
             .set(DEFAULT_CORDIC_FREQ)
             .set_coercer(boost::bind(&tx_dsp_core_3000::set_freq, this, _1))
+            .set_publisher([this](){ return this->get_freq(); })
         ;
         subtree->create<meta_range_t>("freq/range")
             .set_publisher(boost::bind(&tx_dsp_core_3000::get_freq_range, this))
@@ -200,6 +197,7 @@ private:
     const size_t _dsp_base;
     double _tick_rate, _link_rate;
     double _scaling_adjustment, _dsp_extra_scaling, _host_extra_scaling, _fxpt_scalar_correction;
+    double _current_freq;
 };
 
 tx_dsp_core_3000::sptr tx_dsp_core_3000::make(wb_iface::sptr iface, const size_t dsp_base)
