@@ -1,25 +1,16 @@
 //
 // Copyright 2014-15 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "twinrx/twinrx_experts.hpp"
 #include "twinrx/twinrx_ctrl.hpp"
 #include "twinrx/twinrx_io.hpp"
 #include "twinrx/twinrx_ids.hpp"
-#include <expert_factory.hpp>
+
+#include <uhdlib/experts/expert_factory.hpp>
 #include <uhd/types/device_addr.hpp>
 #include <uhd/types/ranges.hpp>
 #include <uhd/types/sensors.hpp>
@@ -28,7 +19,6 @@
 #include <uhd/utils/log.hpp>
 #include <uhd/utils/static.hpp>
 #include "dboard_ctor_args.hpp"
-#include <boost/assign/list_of.hpp>
 #include <boost/make_shared.hpp>
 #include <boost/thread.hpp>
 #include <boost/thread/mutex.hpp>
@@ -109,8 +99,8 @@ public:
         expert_factory::add_dual_prop_node<double>(_expert, get_rx_subtree(),
             "los/LO2/freq/value", prepend_ch("los/LO2/freq/desired", _ch_name), prepend_ch("los/LO2/freq/coerced", _ch_name),
             0.0, AUTO_RESOLVE_ON_READ_WRITE);
-        get_rx_subtree()->create<std::vector<std::string> >("los/all/source/options")
-            .set(boost::assign::list_of("internal")("external")("companion")("disabled")("reimport"));
+        get_rx_subtree()->create<std::vector<std::string>>("los/all/source/options")
+            .set({"internal", "external", "companion", "disabled", "reimport"});
         expert_factory::add_prop_node<std::string>(_expert, get_rx_subtree(),
             "los/all/source/value", prepend_ch("los/all/source", _ch_name),
             "internal", AUTO_RESOLVE_ON_WRITE);
@@ -120,19 +110,19 @@ public:
 
         //Gain Specific
         get_rx_subtree()->create<meta_range_t>("gains/all/range")
-            .set(gain_range_t(0, 95, double(1.0)));
+            .set(gain_range_t(0, 93, double(1.0)));
         expert_factory::add_prop_node<double>(_expert, get_rx_subtree(),
             "gains/all/value", prepend_ch("gain", _ch_name),
             0.0, AUTO_RESOLVE_ON_WRITE);
         get_rx_subtree()->create<std::vector<std::string> >("gains/all/profile/options")
-            .set(boost::assign::list_of("low-noise")("low-distortion")("default"));
+            .set({"low-noise", "low-distortion", "default"});
         expert_factory::add_prop_node<std::string>(_expert, get_rx_subtree(),
             "gains/all/profile/value", prepend_ch("gain_profile", _ch_name),
             "default", AUTO_RESOLVE_ON_WRITE);
 
         //Antenna Specific
         get_rx_subtree()->create<std::vector<std::string> >("antenna/options")
-            .set(boost::assign::list_of("RX1")("RX2"));
+            .set({"RX1", "RX2"});
         expert_factory::add_prop_node<std::string>(_expert, get_rx_subtree(),
             "antenna/value", prepend_ch("antenna", _ch_name),
             (_ch_name == "0" ? "RX1" : "RX2"), AUTO_RESOLVE_ON_WRITE);
@@ -263,7 +253,7 @@ public:
         // Add workers to expert
         //---------------------------------------------------------
         //Channel (front-end) specific
-        BOOST_FOREACH(const std::string& fe, _fe_names) {
+        for(const std::string& fe:  _fe_names) {
             expert_factory::add_worker_node<twinrx_freq_path_expert>(_expert, _expert->node_retriever(), fe);
             expert_factory::add_worker_node<twinrx_freq_coercion_expert>(_expert, _expert->node_retriever(), fe);
             expert_factory::add_worker_node<twinrx_chan_gain_expert>(_expert, _expert->node_retriever(), fe);
@@ -336,7 +326,7 @@ UHD_STATIC_BLOCK(reg_twinrx_dboards)
         twinrx::TWINRX_REV_A_ID,
         &twinrx_rcvr::make_twinrx_fe,
         "TwinRX Rev A",
-        boost::assign::list_of("0")("1"),
+        {"0", "1"},
         &make_twinrx_container
     );
 
@@ -344,7 +334,7 @@ UHD_STATIC_BLOCK(reg_twinrx_dboards)
         twinrx::TWINRX_REV_B_ID,
         &twinrx_rcvr::make_twinrx_fe,
         "TwinRX Rev B",
-        boost::assign::list_of("0")("1"),
+        {"0", "1"},
         &make_twinrx_container
     );
 
@@ -352,7 +342,7 @@ UHD_STATIC_BLOCK(reg_twinrx_dboards)
         twinrx::TWINRX_REV_C_ID,
         &twinrx_rcvr::make_twinrx_fe,
         "TwinRX Rev C",
-        boost::assign::list_of("0")("1"),
+        {"0", "1"},
         &make_twinrx_container
     );
 }
