@@ -94,6 +94,7 @@ void magnesium_radio_ctrl_impl::_init_defaults()
         radio_ctrl_impl::set_tx_frequency(MAGNESIUM_DEFAULT_FREQ, chan);
         radio_ctrl_impl::set_tx_gain(0, chan);
         radio_ctrl_impl::set_tx_antenna(MAGNESIUM_DEFAULT_TX_ANTENNA, chan);
+        radio_ctrl_impl::set_tx_bandwidth(MAGNESIUM_DEFAULT_BANDWIDTH, chan);
     }
 
 
@@ -111,60 +112,42 @@ void magnesium_radio_ctrl_impl::_init_defaults()
 void magnesium_radio_ctrl_impl::_init_peripherals()
 {
     UHD_LOG_TRACE(unique_id(), "Initializing peripherals...");
-    fs_path cpld_path  = _root_path.branch_path()
-        / str(boost::format("Radio_%d") % ((get_block_id().get_block_count()/2)*2))
-        / "cpld";
-    fs_path rx_lo_path  = _root_path.branch_path()
-        / str(boost::format("Radio_%d") % ((get_block_id().get_block_count()/2)*2))
-        / "rx_lo";
-    fs_path tx_lo_path  = _root_path.branch_path()
-        / str(boost::format("Radio_%d") % ((get_block_id().get_block_count()/2)*2))
-        / "tx_lo";
     UHD_LOG_TRACE(unique_id(), "Initializing SPI core...");
     _spi = spi_core_3000::make(_get_ctrl(0),
         regs::sr_addr(regs::SPI),
         regs::rb_addr(regs::RB_SPI)
     );
-
     UHD_LOG_TRACE(unique_id(), "Initializing CPLD...");
-    UHD_LOG_TRACE(unique_id(), "CPLD path: " << cpld_path);
-    if (not _tree->exists(cpld_path)) {
-        UHD_LOG_TRACE(unique_id(), "Creating new CPLD object...");
-        spi_config_t spi_config;
-        spi_config.use_custom_divider = true;
-        spi_config.divider = 125;
-        spi_config.mosi_edge = spi_config_t::EDGE_RISE;
-        spi_config.miso_edge = spi_config_t::EDGE_FALL;
-        UHD_LOG_TRACE(unique_id(), "Making CPLD object...");
-        _cpld = std::make_shared<magnesium_cpld_ctrl>(
-            [this, spi_config](const uint32_t transaction){ // Write functor
-                this->_spi->write_spi(
-                    SEN_CPLD,
-                    spi_config,
-                    transaction,
-                    24
-                );
-            },
-            [this, spi_config](const uint32_t transaction){ // Read functor
-                return this->_spi->read_spi(
-                    SEN_CPLD,
-                    spi_config,
-                    transaction,
-                    24
-                );
-            }
-        );
-        _update_atr_switches(
-            magnesium_cpld_ctrl::BOTH,
-            DX_DIRECTION,
-            radio_ctrl_impl::get_rx_antenna(0)
-        );
-        _tree->create<magnesium_cpld_ctrl::sptr>(cpld_path).set(_cpld);
-    } else {
-        UHD_LOG_TRACE(unique_id(), "Reusing someone else's CPLD object...");
-        _cpld = _tree->access<magnesium_cpld_ctrl::sptr>(cpld_path).get();
-    }
-
+    UHD_LOG_TRACE(unique_id(), "Creating new CPLD object...");
+    spi_config_t spi_config;
+    spi_config.use_custom_divider = true;
+    spi_config.divider = 125;
+    spi_config.mosi_edge = spi_config_t::EDGE_RISE;
+    spi_config.miso_edge = spi_config_t::EDGE_FALL;
+    UHD_LOG_TRACE(unique_id(), "Making CPLD object...");
+    _cpld = std::make_shared<magnesium_cpld_ctrl>(
+        [this, spi_config](const uint32_t transaction){ // Write functor
+            this->_spi->write_spi(
+                SEN_CPLD,
+                spi_config,
+                transaction,
+                24
+            );
+        },
+        [this, spi_config](const uint32_t transaction){ // Read functor
+            return this->_spi->read_spi(
+                SEN_CPLD,
+                spi_config,
+                transaction,
+                24
+            );
+        }
+    );
+    _update_atr_switches(
+        magnesium_cpld_ctrl::BOTH,
+        DX_DIRECTION,
+        radio_ctrl_impl::get_rx_antenna(0)
+    );
     UHD_LOG_TRACE(unique_id(), "Initializing TX LO...");
     _tx_lo = adf435x_iface::make_adf4351(
         [this](const std::vector<uint32_t> transactions){
@@ -879,4 +862,3 @@ void magnesium_radio_ctrl_impl::_init_mpm_sensors(
         ;
     }
 }
-
