@@ -1,5 +1,5 @@
 //
-// Copyright 2014 Ettus Research LLC
+// Copyright 2014-2016 Ettus Research LLC
 // Copyright 2018 Ettus Research, a National Instruments Company
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
@@ -164,7 +164,7 @@ void device3_impl::enumerate_rfnoc_blocks(
             ));
             uhd::rfnoc::ctrl_iface::sptr ctrl1 = uhd::rfnoc::ctrl_iface::make(
                     xport1,
-                    str(boost::format("CE_%02d_Port_%02d") % i % ctrl_sid.get_dst_endpoint())
+                    str(boost::format("CE_%02d_Port_%02X") % i % ctrl_sid.get_dst_endpoint())
             );
             make_args.ctrl_ifaces[port_number] = ctrl1;
         }
