@@ -1,24 +1,18 @@
 //
-// Copyright 2013-2014 Ettus Research LLC
+// Copyright 2013-2017 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifdef E300_NATIVE
 
-#include <stdint.h>
+#include <uhdlib/utils/system_time.hpp>
 #include <uhd/config.hpp>
+#include <stdint.h>
+#include <atomic>
+#include <chrono>
+#include <thread>
 
 // constants coded into the fpga parameters
 static const size_t ZF_CONFIG_BASE    = 0x40000000;
@@ -81,12 +75,12 @@ static UHD_INLINE size_t ZF_STREAM_OFF(const size_t which)
 #include <fcntl.h> //open, close
 #include <poll.h> //poll
 #include <uhd/utils/log.hpp>
-#include <uhd/utils/msg.hpp>
+
 #include <boost/format.hpp>
 #include <boost/thread/thread.hpp> //sleep
 #include <uhd/types/time_spec.hpp> //timeout
 #include <uhd/utils/log.hpp>
-#include <uhd/utils/atomic.hpp>
+#include <uhdlib/utils/atomic.hpp>
 
 //locking stuff for shared irq
 #include <boost/thread/mutex.hpp>
@@ -101,6 +95,10 @@ struct e300_fifo_poll_waiter
         //NOP
     }
 
+    /*!
+     * Waits until the file descriptor fd has data to read.
+     * Access to the file descriptor is thread safe.
+     */
     void wait(const double timeout)
     {
         if (timeout == 0) {
@@ -110,7 +108,7 @@ struct e300_fifo_poll_waiter
         boost::mutex::scoped_lock l(_mutex);
         if (_poll_claimed)
         {
-            _cond.timed_wait(l, boost::posix_time::microseconds(timeout*1000000));
+            _cond.timed_wait(l, boost::posix_time::microseconds(long(timeout*1000000)));
         }
         else
         {
@@ -210,9 +208,9 @@ public:
         _index(0),
         _waiter(waiter)
     {
-        //UHD_MSG(status) << boost::format("phys 0x%x") % addrs.phys << std::endl;
-        //UHD_MSG(status) << boost::format("data 0x%x") % addrs.data << std::endl;
-        //UHD_MSG(status) << boost::format("ctrl 0x%x") % addrs.ctrl << std::endl;
+        //UHD_LOGGER_INFO("E300") << boost::format("phys 0x%x") % addrs.phys ;
+        //UHD_LOGGER_INFO("E300") << boost::format("data 0x%x") % addrs.data ;
+        //UHD_LOGGER_INFO("E300") << boost::format("ctrl 0x%x") % addrs.ctrl ;
 
         const uint32_t sig = zf_peek32(_addrs.ctrl + ARBITER_RD_SIG);
         UHD_ASSERT_THROW((sig >> 16) == 0xACE0);
@@ -243,7 +241,7 @@ public:
     template <typename T>
     UHD_INLINE typename T::sptr get_buff(const double timeout)
     {
-        const time_spec_t exit_time = time_spec_t::get_system_time() + time_spec_t(timeout);
+        const time_spec_t exit_time = uhd::get_system_time() + time_spec_t(timeout);
         while (1)
         {
             if (zf_peek32(_addrs.ctrl + ARBITER_RB_STATUS_OCC))
@@ -256,11 +254,11 @@ public:
                     _index = 0;
                 return _buffs[_index++]->get_new<T>();
             }
-            if (time_spec_t::get_system_time() > exit_time) {
+            if (uhd::get_system_time() > exit_time) {
                 break;
             }
             _waiter->wait(timeout);
-            //boost::this_thread::sleep(boost::posix_time::milliseconds(1));
+            //std::this_thread::sleep_for(std::chrono::milliseconds(1));
         }
 
         return typename T::sptr();
@@ -351,7 +349,7 @@ public:
     virtual ~e300_fifo_interface_impl(void)
     {
         delete _waiter;
-        UHD_LOG << "cleanup: munmap" << std::endl;
+        UHD_LOGGER_TRACE("E300")<< "cleanup: munmap" ;
         ::munmap(_buff, _config.ctrl_length + _config.buff_length);
         ::close(_fd);
     }
