@@ -1,18 +1,8 @@
 //
 // Copyright 2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifndef INCLUDED_E300_GLOBAL_REGS_HPP
@@ -43,6 +33,7 @@ public:
     static const size_t SR_CORE_MISC     = 4;
     static const size_t SR_CORE_TEST     = 28;
     static const size_t SR_CORE_XB_LOCAL = 32;
+    static const size_t SR_CORE_SPI_SEL  = 64;
 
     // leave some room for registers,
     // xbar starts with an offset of one
@@ -56,6 +47,7 @@ public:
     static const size_t RB32_CORE_COMPAT  = 2;
     static const size_t RB32_CORE_GITHASH = 3;
     static const size_t RB32_CORE_PLL     = 4;
+    static const size_t RB32_CORE_NUM_CE  = 8;
     static const size_t RB32_CORE_TEST    = 24;
 
     // PPS selection
