@@ -1,18 +1,8 @@
 //
 // Copyright 2013-2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "e300_network.hpp"
@@ -21,8 +11,6 @@
 
 #include "e300_impl.hpp"
 
-#include "ad9361_ctrl.hpp"
-
 #include "e300_sensor_manager.hpp"
 #include "e300_fifo_config.hpp"
 #include "e300_spi.hpp"
@@ -31,16 +19,19 @@
 #include "e300_common.hpp"
 #include "e300_remote_codec_ctrl.hpp"
 
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <uhd/utils/byteswap.hpp>
 #include <uhd/utils/paths.hpp>
 
+#include <uhdlib/usrp/common/ad9361_ctrl.hpp>
+
 #include <boost/asio.hpp>
-#include <boost/thread.hpp>
 #include <boost/filesystem.hpp>
 #include <boost/make_shared.hpp>
 
 #include <fstream>
+#include <chrono>
+#include <thread>
 
 using namespace uhd;
 using namespace uhd::transport;
@@ -88,7 +79,7 @@ static void e300_recv_tunnel(
             //step 1 - get the buffer
             managed_recv_buffer::sptr buff = recver->get_recv_buff();
             if (not buff) continue;
-            if (E300_NETWORK_DEBUG) UHD_MSG(status) << name << " got " << buff->size() << std::endl;
+            if (E300_NETWORK_DEBUG) UHD_LOGGER_INFO("E300") << name << " got " << buff->size();
 
             //step 1.5 -- update endpoint
             {
@@ -102,13 +93,13 @@ static void e300_recv_tunnel(
     }
     catch(const std::exception &ex)
     {
-        UHD_MSG(error) << "e300_recv_tunnel exit " << name << " " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_recv_tunnel exit " << name << " " << ex.what();
     }
     catch(...)
     {
-        UHD_MSG(error) << "e300_recv_tunnel exit " << name << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_recv_tunnel exit " << name ;
     }
-    UHD_MSG(status) << "e300_recv_tunnel exit " << name << std::endl;
+    UHD_LOGGER_INFO("E300") << "e300_recv_tunnel exit " << name;
     *running = false;
 }
 
@@ -133,10 +124,10 @@ static void e300_send_tunnel(
             if (not buff) continue;
 
             //step 2 - recv from socket
-            while (not wait_for_recv_ready(recver->native(), 100) and *running){}
+            while (not wait_for_recv_ready(recver->native_handle(), 100) and *running){}
             if (not *running) break;
             const size_t num_bytes = recver->receive_from(asio::buffer(buff->cast<void *>(), buff->size()), _rx_endpoint);
-            if (E300_NETWORK_DEBUG) UHD_MSG(status) << name << " got " << num_bytes << std::endl;
+            if (E300_NETWORK_DEBUG) UHD_LOGGER_INFO("E300") << name << " got " << num_bytes;
 
             //step 2.5 -- update endpoint
             {
@@ -150,13 +141,13 @@ static void e300_send_tunnel(
     }
     catch(const std::exception &ex)
     {
-        UHD_MSG(error) << "e300_send_tunnel exit " << name << " " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_send_tunnel exit " << name << " " << ex.what() ;
     }
     catch(...)
     {
-        UHD_MSG(error) << "e300_send_tunnel exit " << name << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_send_tunnel exit " << name ;
     }
-    UHD_MSG(status) << "e300_send_tunnel exit " << name << std::endl;
+    UHD_LOGGER_INFO("E300") << "e300_send_tunnel exit " << name;
     *running = false;
 }
 
@@ -252,7 +243,7 @@ static void e300_codec_ctrl_tunnel(
                 out->bw = _codec_ctrl->set_bw_filter(which_str, in->bw);
                 break;
             default:
-                UHD_MSG(status) << "Got unknown request?!" << std::endl;
+                UHD_LOGGER_INFO("E300") << "Got unknown request?!";
                 //Zero out actions to fail this request on client
                 out->action = uhd::htonx<uint32_t>(0);
             }
@@ -262,13 +253,13 @@ static void e300_codec_ctrl_tunnel(
     }
     catch(const std::exception &ex)
     {
-        UHD_MSG(error) << "e300_ctrl_tunnel exit " << name << " " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_ctrl_tunnel exit " << name << " " << ex.what() ;
     }
     catch(...)
     {
-        UHD_MSG(error) << "e300_ctrl_tunnel exit " << name << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_ctrl_tunnel exit " << name ;
     }
-    UHD_MSG(status) << "e300_ctrl_tunnel exit " << name << std::endl;
+    UHD_LOGGER_INFO("E300") << "e300_ctrl_tunnel exit " << name;
     *running = false;
 }
 
@@ -309,13 +300,13 @@ static void e300_global_regs_tunnel(
     }
     catch(const std::exception &ex)
     {
-        UHD_MSG(error) << "e300_gregs_tunnel exit " << name << " " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_gregs_tunnel exit " << name << " " << ex.what() ;
     }
     catch(...)
     {
-        UHD_MSG(error) << "e300_gregs_tunnel exit " << name << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_gregs_tunnel exit " << name ;
     }
-    UHD_MSG(status) << "e300_gregs_tunnel exit " << name << std::endl;
+    UHD_LOGGER_INFO("E300") << "e300_gregs_tunnel exit " << name;
     *running = false;
 }
 
@@ -353,20 +344,20 @@ static void e300_sensor_tunnel(
                 in->value = uhd::htonx<uint32_t>(
                     sensor_manager->get_ref_lock().to_bool() ? 1 : 0);
             } else
-                UHD_MSG(status) << "Got unknown request?!" << std::endl;
+                UHD_LOGGER_INFO("E300") << "Got unknown request?!";
 
             socket->send_to(asio::buffer(in_buff, sizeof(sensor_transaction_t)), *endpoint);
         }
     }
     catch(const std::exception &ex)
     {
-        UHD_MSG(error) << "e300_sensor_tunnel exit " << name << " " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_sensor_tunnel exit " << name << " " << ex.what() ;
     }
     catch(...)
     {
-        UHD_MSG(error) << "e300_sensor_tunnel exit " << name << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_sensor_tunnel exit " << name ;
     }
-    UHD_MSG(status) << "e300_sensor_tunnel exit " << name << std::endl;
+    UHD_LOGGER_INFO("E300") << "e300_sensor_tunnel exit " << name;
     *running = false;
 }
 
@@ -419,19 +410,19 @@ static void e300_i2c_tunnel(
                 }
 
             } else {
-                UHD_MSG(error) << "e300_i2c_tunnel could not handle message." << std::endl;
+                UHD_LOGGER_ERROR("E300") << "e300_i2c_tunnel could not handle message." ;
             }
         }
     }
     catch(const std::exception &ex)
     {
-        UHD_MSG(error) << "e300_i2c_tunnel exit " << name << " " << ex.what() << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_i2c_tunnel exit " << name << " " << ex.what() ;
     }
     catch(...)
     {
-        UHD_MSG(error) << "e300_i2c_tunnel exit " << name << std::endl;
+        UHD_LOGGER_ERROR("E300") << "e300_i2c_tunnel exit " << name ;
     }
-    UHD_MSG(status) << "e300_i2c_tunnel exit " << name << std::endl;
+    UHD_LOGGER_INFO("E300") << "e300_i2c_tunnel exit " << name;
     *running = false;
 }
 
@@ -491,7 +482,7 @@ void network_server_impl::_run_server(
     //boost::shared_ptr<asio::ip::udp::acceptor> acceptor(new asio::ip::udp::acceptor(io_service, endpoint));
     while (not boost::this_thread::interruption_requested())
     {
-        UHD_MSG(status) << "e300 run server on port " << port << " for " << what << std::endl;
+        UHD_LOGGER_INFO("E300") << "e300 run server on port " << port << " for " << what;
         try
         {
             //while (not wait_for_recv_ready(acceptor->native(), 100))
@@ -501,7 +492,7 @@ void network_server_impl::_run_server(
             boost::shared_ptr<asio::ip::udp::socket> socket;
             socket.reset(new asio::ip::udp::socket(io_service, endpoint));
             //acceptor->accept(*socket);
-            UHD_MSG(status) << "e300 socket accept on port " << port << " for " << what << std::endl;
+            UHD_LOGGER_INFO("E300") << "e300 socket accept on port " << port << " for " << what;
             //asio::ip::udp::no_delay option(true);
             //socket->set_option(option);
             boost::thread_group tg;
@@ -586,8 +577,8 @@ network_server_impl::network_server_impl(const uhd::device_addr_t &device_addr)
                 break;
             case e300_eeprom_manager::UNKNOWN:
             default:
-                UHD_MSG(warning) << "Unknown motherboard type, loading e300 image."
-                                 << std::endl;
+                UHD_LOGGER_WARNING("E300") << "Unknown motherboard type, loading e300 image."
+                                 ;
                 fpga_image = find_image_path(E300_FPGA_FILE_NAME);
                 break;
             }
@@ -640,7 +631,7 @@ network_server_impl::network_server_impl(const uhd::device_addr_t &device_addr)
     ad9361_params::sptr client_settings = boost::make_shared<e300_ad9361_client_t>();
     _codec_ctrl = ad9361_ctrl::make_spi(client_settings, spi::make(E300_SPIDEV_DEVICE), 1);
     // This is horrible ... why do I have to sleep here?
-    boost::this_thread::sleep(boost::posix_time::milliseconds(100));
+    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     _sensor_manager = e300_sensor_manager::make_local(_global_regs);
 }
 
