@@ -1,25 +1,15 @@
 //
 // Copyright 2014 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #ifndef INCLUDED_E300_REMOTE_CODEC_CTRL_HPP
 #define INCLUDED_E300_REMOTE_CODEC_CTRL_HPP
 
-#include "ad9361_ctrl.hpp"
 #include <uhd/transport/zero_copy.hpp>
+#include <uhdlib/usrp/common/ad9361_ctrl.hpp>
 
 namespace uhd { namespace usrp { namespace e300 {
 
