@@ -1,29 +1,18 @@
 //
 // Copyright 2013 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include "x300_impl.hpp"
 #include <uhd/types/wb_iface.hpp>
 #include "x300_regs.hpp"
-#include <uhd/utils/msg.hpp>
+#include <uhd/utils/log.hpp>
 #include <uhd/types/serial.hpp>
 #include <uhd/exception.hpp>
 #include <boost/format.hpp>
-#include <boost/foreach.hpp>
-#include <boost/thread/thread.hpp>
+#include <chrono>
 
 using namespace uhd;
 
@@ -64,8 +53,8 @@ struct x300_uart_iface : uart_iface
 
     void write_uart(const std::string &buff)
     {
-        boost::mutex::scoped_lock(_write_mutex);
-        BOOST_FOREACH(const char ch, buff)
+        boost::mutex::scoped_lock lock(_write_mutex);
+        for(const char ch:  buff)
         {
             this->putchar(ch);
         }
@@ -136,8 +125,11 @@ struct x300_uart_iface : uart_iface
 
     std::string read_uart(double timeout)
     {
-        boost::mutex::scoped_lock(_read_mutex);
-        const boost::system_time exit_time = boost::get_system_time() + boost::posix_time::microseconds(long(timeout*1e6));
+        boost::mutex::scoped_lock lock(_read_mutex);
+        const auto exit_time =
+            std::chrono::steady_clock::now()
+            + std::chrono::microseconds(int64_t(timeout*1e6));
+
         std::string buff;
 
         while (true)
@@ -160,8 +152,9 @@ struct x300_uart_iface : uart_iface
             }
 
             // no more characters - check time
-            if (boost::get_system_time() > exit_time)
+            if (std::chrono::steady_clock::now() > exit_time) {
                 break;
+            }
         }
 
         return buff;
