@@ -1,18 +1,8 @@
 #
 # Copyright 2015 Ettus Research LLC
+# Copyright 2018 Ettus Research, a National Instruments Company
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-or-later
 #
 """
 Run device tests for the E3XX series.
@@ -40,7 +30,6 @@ uhd_benchmark_rate_test.tests = {
         'chan': '1',
         'rate': 1e6,
         'acceptable-underruns': 50,
-        'products': ['B210',],
     },
 }
 
