@@ -1,23 +1,14 @@
 //
 // Copyright 2012,2014-2016 Ettus Research LLC
+// Copyright 2018 Ettus Research, a National Instruments Company
 //
-// This program is free software: you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
-// (at your option) any later version.
-//
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <http://www.gnu.org/licenses/>.
+// SPDX-License-Identifier: GPL-3.0-or-later
 //
 
 #include <uhd/utils/paths.hpp>
-#include <uhd/utils/thread_priority.hpp>
+#include <uhd/utils/thread.hpp>
 #include <uhd/utils/safe_main.hpp>
+#include <uhd/utils/algorithm.hpp>
 #include <uhd/usrp/multi_usrp.hpp>
 #include <uhd/usrp_clock/multi_usrp_clock.hpp>
 #include <boost/filesystem.hpp>
@@ -25,17 +16,16 @@
 #include <boost/format.hpp>
 #include <iostream>
 #include <complex>
-#include <boost/thread.hpp>
 #include <string>
 #include <cmath>
 #include <ctime>
 #include <cstdlib>
+#include <chrono>
+#include <thread>
 
 namespace po = boost::program_options;
 namespace fs = boost::filesystem;
 
-const size_t WARMUP_TIMEOUT_MS = 30000;
-
 void print_notes(void) {
   // Helpful notes
   std::cout << boost::format("**************************************Helpful Notes on Clock/PPS Selection**************************************\n");
@@ -64,7 +54,7 @@ int query_clock_sensors(const std::string &args) {
       std::cout << boost::format("%s\n%s\n") % gga_string.to_pp_string() % rmc_string.to_pp_string();
       std::cout << boost::format("\nPrinting GPS servo status:\n");
       std::cout << boost::format("%s\n\n") % servo_string.to_pp_string();
-  } catch (uhd::lookup_error &e) {
+  } catch (const uhd::lookup_error &) {
       std::cout << "NMEA strings not implemented for this device." << std::endl;
   }
   std::cout << boost::format("GPS Epoch time: %.5f seconds\n") % clock->get_sensor("gps_time").to_real();
@@ -122,71 +112,67 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
       return EXIT_FAILURE;
   }
 
-  std::cout << "\nSetting the reference clock source to \"gpsdo\"...\n";
-  try {
-      usrp->set_clock_source("gpsdo");
-  } catch (uhd::value_error &e) {
-      std::cout << "could not set the clock source to \"gpsdo\"; error was:" <<std::endl;
-      std::cout << e.what() << std::endl;
-      std::cout << "trying \"external\"..." <<std::endl;
-      try{
-          usrp->set_clock_source("external");
-      } catch (uhd::value_error&) {
-          std::cout << "\"external\" failed, too." << std::endl;
-      }
-  }
-  std::cout<< std::endl << "Clock source is now " << usrp->get_clock_source(0) << std::endl;
+  bool ref_set_to_gpsdo = false;
 
-  //Check for 10 MHz lock
+  // Set clock source to gpsdo if supported
+  if (uhd::has(usrp->get_clock_sources(0),"gpsdo"))
+  {
+      std::cout << "Setting the reference clock source to \"gpsdo\"..." << std::endl;
+      usrp->set_clock_source("gpsdo");
+      ref_set_to_gpsdo = true;
+  }
+  std::cout << "Clock source is now " << usrp->get_clock_source(0) << std::endl;
+
+  // Set time source to gpsdo if supported
+  if (uhd::has(usrp->get_time_sources(0),"gpsdo"))
+  {
+      std::cout << "Setting the reference clock source to \"gpsdo\"..." << std::endl;
+      usrp->set_time_source("gpsdo");
+      ref_set_to_gpsdo = true;
+  }
+  std::cout << "Time source is now " << usrp->get_time_source(0) << std::endl;
+
+  if (not ref_set_to_gpsdo)
+  {
+      std::cerr << "ERROR: Unable to set clock or time reference to \"gpsdo\"" << std::endl;
+      return EXIT_FAILURE;
+  }
+
+  //Check for ref lock
   if(std::find(sensor_names.begin(), sensor_names.end(), "ref_locked") != sensor_names.end()) {
       uhd::sensor_value_t ref_locked = usrp->get_mboard_sensor("ref_locked",0);
-      for (size_t i = 0; not ref_locked.to_bool() and i < 100; i++) {
-          boost::this_thread::sleep(boost::posix_time::milliseconds(100));
+      for (size_t i = 0; not ref_locked.to_bool() and i < 300; i++) {
+          std::this_thread::sleep_for(std::chrono::milliseconds(100));
           ref_locked = usrp->get_mboard_sensor("ref_locked",0);
       }
       if(not ref_locked.to_bool()) {
-          std::cout << boost::format("USRP NOT Locked to GPSDO 10 MHz Reference.\n");
+          std::cout << boost::format("USRP NOT Locked to Reference.\n");
           std::cout << boost::format("Double check installation instructions (N2X0/E1X0 only): https://www.ettus.com/content/files/gpsdo-kit_4.pdf\n\n");
           return EXIT_FAILURE;
       } else {
-          std::cout << boost::format("USRP Locked to GPSDO 10 MHz Reference.\n");
+          std::cout << boost::format("USRP Locked to Reference.\n");
       }
   } else {
       std::cout << boost::format("ref_locked sensor not present on this board.\n");
   }
 
-  // Explicitly set time source to gpsdo
-  try {
-      usrp->set_time_source("gpsdo");
-  } catch (uhd::value_error &e) {
-      std::cout << "could not set the time source to \"gpsdo\"; error was:" <<std::endl;
-      std::cout << e.what() << std::endl;
-      std::cout << "trying \"external\"..." <<std::endl;
-      try {
-          usrp->set_time_source("external");
-      } catch (uhd::value_error&) {
-          std::cout << "\"external\" failed, too." << std::endl;
-      }
-  }
-  std::cout << std::endl << "Time source is now " << usrp->get_time_source(0) << std::endl;
-
   print_notes();
 
-  // The TCXO has a long warm up time, so wait up to 30 seconds for sensor data to show up
+  // The TCXO has a long warm up time, so wait up to 30 seconds for sensor data
+  // to show up
   std::cout << "Waiting for the GPSDO to warm up..." << std::flush;
-  boost::system_time exit_time = boost::get_system_time() +
-              boost::posix_time::milliseconds(WARMUP_TIMEOUT_MS);
-  while (boost::get_system_time() < exit_time) {
+  auto end = std::chrono::steady_clock::now() + std::chrono::seconds(30);
+  while (std::chrono::steady_clock::now() < end) {
       try {
-          usrp->get_mboard_sensor("gps_locked",0);
+          usrp->get_mboard_sensor("gps_locked", 0);
           break;
       } catch (std::exception &) {}
-      boost::this_thread::sleep(boost::posix_time::milliseconds(100));
+      std::this_thread::sleep_for(std::chrono::milliseconds(250));
       std::cout << "." << std::flush;
   }
   std::cout << std::endl;
   try {
-      usrp->get_mboard_sensor("gps_locked",0);
+      usrp->get_mboard_sensor("gps_locked", 0);
   } catch (std::exception &) {
       std::cout << "No response from GPSDO in 30 seconds" << std::endl;
       return EXIT_FAILURE;
@@ -213,12 +199,15 @@ int UHD_SAFE_MAIN(int argc, char *argv[]){
   if(pps_seconds != gps_seconds) {
       std::cout << "\nTrying to align the device time to GPS time..."
                 << std::endl;
+
+      gps_time = usrp->get_mboard_sensor("gps_time");
+
       //set the device time to the GPS time
       //getting the GPS time returns just after the PPS edge, so just add a
       //second and set the device time at the next PPS edge
       usrp->set_time_next_pps(uhd::time_spec_t(gps_time.to_int() + 1.0));
       //allow some time to make sure the PPS has come…
-      boost::this_thread::sleep(boost::posix_time::milliseconds(1100));
+      std::this_thread::sleep_for(std::chrono::milliseconds(1100));
       //…then ask
       gps_seconds = usrp->get_mboard_sensor("gps_time").to_int();
       pps_seconds = usrp->get_time_last_pps().to_ticks(1.0);
